@@ -18,7 +18,6 @@
  */
 
 #include <config.h>
-#include <dirent.h>
 #include "gnome.h"
 #include "sol.h"
 #include "menu.h"
@@ -29,8 +28,7 @@
 
 static GtkWidget *about = NULL;
 
-
-void restart_game ()
+static void restart_game ()
 {
   if (waiting_for_mouse_up()) return;
   new_game (NULL, &seed);
@@ -42,12 +40,6 @@ void random_seed ()
   new_game (NULL, NULL);
 };
 
-void new_rules (GtkWidget* w, gchar* file) 
-{
-  if (waiting_for_mouse_up()) return;
-  new_game (file, NULL);
-};
-
 void undo_callback ()
 {
   if (waiting_for_mouse_up()) return;
@@ -72,16 +64,19 @@ void help_about_callback ()
 {
   GdkPixbuf *pixbuf = NULL;
   const gchar *authors[] = {
-	  N_("Main program:  Jonathan Blandford (jrb@redhat.com)"),
-	  N_("                      Felix Bellaby (felix@pooh.u-net.com)"),
-	  N_("                      Rosanna Yuen (zana@webwynk.net)"),
-	  N_("Card Games:    Jonathan Blandford (jrb@redhat.com)"),
-	  N_("                      W. Borgert (debacle@debian.org)"),
-	  N_("                      Robert Brady (rwb197@ecs.soton.ac.uk)"),
-	  N_("                      Nick Lamb (njl195@zepler.org.uk)"),
-	  N_("                      Changwoo Ryu (cwryu@adam.kaist.ac.kr)"),
-          N_("                      Matthew Wilcox (matthew@wil.cx)"),
-	  N_("                      Rosanna Yuen (zana@webwynk.net)"),
+	  N_("Main game:"),
+	  "Jonathan Blandford (jrb@redhat.com)",
+	  "Felix Bellaby (felix@pooh.u-net.com)",
+	  "Rosanna Yuen (zana@webwynk.net)",
+	  "",
+	  N_("Card games:"),
+	  "Jonathan Blandford (jrb@redhat.com)",
+	  "W. Borgert (debacle@debian.org)",
+	  "Robert Brady (rwb197@ecs.soton.ac.uk)",
+	  "Nick Lamb (njl195@zepler.org.uk)",
+	  "Changwoo Ryu (cwryu@adam.kaist.ac.kr)",
+	  "Matthew Wilcox (matthew@wil.cx)",
+	  "Rosanna Yuen (zana@webwynk.net)",
 	  NULL
   };
 
@@ -112,9 +107,10 @@ void help_about_callback ()
   }
   about = gnome_about_new ( _("AisleRiot"), VERSION,
 			    /* copyright notice */
-			    _("(C) 1998 Jonathan Blandford (jrb@redhat.com)"),
-			    _("AisleRiot provides a rule-based "
-			    "solitaire card engine that allows many different games to be played"),
+			    "Copyright \xc2\xa9 1998-2003 Jonathan Blandford",
+			    _("AisleRiot provides a rule-based solitaire card "
+			      "engine that allows many different games to be "
+			      "played."),
 			    (const char **)authors,
 			    (const char **)documenters,
 			    strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
@@ -131,6 +127,41 @@ void help_about_callback ()
   return;
 }
 
+static void toolbar_show (void)
+{
+  GtkWidget * toolbar;
+
+  toolbar = GTK_WIDGET(gnome_app_get_dock_item_by_name (GNOME_APP(app),
+                                                        GNOME_APP_TOOLBAR_NAME));
+  gtk_widget_show (toolbar);
+}
+
+void toolbar_hide (void)
+{
+  GtkWidget * toolbar;
+
+  toolbar = GTK_WIDGET(gnome_app_get_dock_item_by_name (GNOME_APP(app),
+                                                        GNOME_APP_TOOLBAR_NAME));
+  gtk_widget_hide (toolbar);
+}
+
+static void toolbar_toggle_callback(GtkWidget * togglebutton, gpointer data)
+{
+  gboolean state;
+  
+  state = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (togglebutton));
+
+  if (state) {
+    toolbar_show();
+    gconf_client_set_bool (gconf_client, "/apps/aisleriot/show_toolbar", TRUE,
+                           NULL);
+  } else {
+    toolbar_hide();
+    gconf_client_set_bool (gconf_client, "/apps/aisleriot/show_toolbar", FALSE,
+                           NULL);
+  }
+}
+
 GnomeUIInfo rules_sub_menu[] = {
   GNOMEUIINFO_END
 };
@@ -148,6 +179,10 @@ GnomeUIInfo settings_menu[] = {
 
   GNOMEUIINFO_SEPARATOR,
 #endif
+
+  GNOMEUIINFO_TOGGLEITEM (N_("_Toolbar"), N_("Show or hide the toolbar"),
+                          toolbar_toggle_callback, NULL),
+
   GNOMEUIINFO_MENU_PREFERENCES_ITEM(show_preferences_dialog, NULL),
 
   GNOMEUIINFO_END
@@ -165,10 +200,10 @@ GnomeUIInfo game_menu[] = {
 
   GNOMEUIINFO_MENU_NEW_GAME_ITEM(random_seed, NULL),
 
-  GNOMEUIINFO_ITEM_STOCK (N_("_Select..."), N_("Choose a different variation of solitaire"), show_select_game_dialog, GTK_STOCK_OPEN),
-
   GNOMEUIINFO_MENU_RESTART_GAME_ITEM(restart_game, NULL),
 
+  GNOMEUIINFO_ITEM_STOCK (N_("_Select..."), N_("Select a different game"), show_select_game_dialog, GTK_STOCK_OPEN),
+
   GNOMEUIINFO_SEPARATOR,
 
   GNOMEUIINFO_MENU_UNDO_MOVE_ITEM(undo_callback, NULL),
@@ -206,33 +241,50 @@ GnomeUIInfo top_menu[] = {
 
 GnomeUIInfo toolbar[] =
 {
-  GNOMEUIINFO_ITEM_STOCK(N_("New"), N_("Deal a new game"),
+  GNOMEUIINFO_ITEM_STOCK(N_("New"), N_("Start a new game"),
 			 random_seed, GTK_STOCK_NEW),
 
-  GNOMEUIINFO_ITEM_STOCK(N_("Restart"), N_("Start this game over"),
+  GNOMEUIINFO_ITEM_STOCK(N_("Restart"), N_("Restart the game"),
 			 restart_game, GTK_STOCK_REFRESH),
 
-  GNOMEUIINFO_ITEM_STOCK(N_("Select"), N_("Select a new game"),
+  GNOMEUIINFO_ITEM_STOCK(N_("Select"), N_("Select a different game"),
 			 show_select_game_dialog, GTK_STOCK_OPEN),
 
   GNOMEUIINFO_SEPARATOR,
 
-  GNOMEUIINFO_ITEM_STOCK(N_("Hint"), N_("Suggest a move"),
-			 show_hint_dialog, GTK_STOCK_JUMP_TO),
-
   GNOMEUIINFO_ITEM_STOCK(N_("Undo"), N_("Undo the last move"),
 			 undo_callback, GTK_STOCK_UNDO),
 
-  GNOMEUIINFO_ITEM_STOCK(N_("Redo"), N_("Redo the last move"),
+  GNOMEUIINFO_ITEM_STOCK(N_("Redo"), N_("Redo the undone move"),
 			 redo_callback, GTK_STOCK_REDO),
 
+  GNOMEUIINFO_ITEM_STOCK(N_("Hint"), N_("Get a hint for your next move"),
+			 show_hint_dialog, GTK_STOCK_HELP),
+
   GNOMEUIINFO_END
 };
 
+void undo_set_sensitive (gboolean state)
+{
+  gtk_widget_set_sensitive (game_menu[4].widget, state);
+  gtk_widget_set_sensitive (toolbar[4].widget, state);
+}
+
+void redo_set_sensitive (gboolean state)
+{
+  gtk_widget_set_sensitive (game_menu[5].widget, state);
+  gtk_widget_set_sensitive (toolbar[5].widget, state);
+}
+
 void create_menus ()
 {
   gnome_app_create_menus (GNOME_APP(app), top_menu);
   gnome_app_create_toolbar (GNOME_APP(app), toolbar);
+
+  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (settings_menu[0].widget),
+                                  gconf_client_get_bool (gconf_client,
+                                                         "/apps/aisleriot/show_toolbar",
+                                                         NULL));
 }
 
 void install_menu_hints (GnomeApp *app)
