@@ -35,6 +35,7 @@
 #include "events.h"
 #include "draw.h"
 #include <gnome.h>
+#include <gconf/gconf-client.h>
 #include <libgnomeui/gnome-window-icon.h>
 #include "slot.h"
 #include "card.h"
@@ -42,6 +43,7 @@
 #include "menu.h"
 #include "splash.h"
 #include "games-clock.h"
+#include "games-gconf.h"
 
 /*
  * Global Variables
@@ -76,6 +78,7 @@ guint            x_spacing = 5;
 guint            y_spacing = 15;
 guint            x_expanded_offset = 20;
 guint            y_expanded_offset = 20;
+GConfClient      *gconf_client = NULL;
 
 #define DEFAULT_VARIATION "klondike.scm"
 #define GNOME_SESSION_BUG
@@ -98,7 +101,7 @@ gchar* game_file_to_name (const gchar* file)
  * This function assumes that xxx.scm has a HTML help file xxx.html
  * located in the gnome/help/aisleriot/../ dir (nb '_' -> '-' for DocBook).
  */
-gpointer* game_file_to_help_entry (const gchar* file)
+static gpointer* game_file_to_help_entry (const gchar* file)
 {
 #if 0
   GnomeHelpMenuEntry* entry;
@@ -116,6 +119,8 @@ gpointer* game_file_to_help_entry (const gchar* file)
 
   g_string_free(help, TRUE);
   return (gpointer *) entry;
+#else
+  return NULL;
 #endif
 }
 
@@ -123,7 +128,7 @@ void make_title ()
 {
   char *title;
 
-  title = g_strdup_printf ("AisleRiot:  %s  ( %d )", game_name, seed);
+  title = g_strdup_printf (_("AisleRiot: %s (%d)"), game_name, seed);
 
   gtk_window_set_title (GTK_WINDOW (app), title); 
 
@@ -158,6 +163,7 @@ void eval_installed_file (char *file)
                                        GTK_BUTTONS_OK,
                                        message);
 
+    gtk_dialog_set_has_separator (GTK_DIALOG (w), FALSE);
     gtk_dialog_run (GTK_DIALOG(w));
 		gtk_widget_destroy(w);
     g_free (message);
@@ -176,19 +182,11 @@ GnomeUIInfo rules_help[] = {
 static int
 save_state (GnomeClient *client)
 {
-  const gchar *prefix = gnome_client_get_config_prefix (client);
+  gconf_client_set_string (gconf_client, "/apps/aisleriot/game_file",
+                           game_file, NULL);
+  gconf_client_set_string (gconf_client, "/apps/aisleriot/card_options",
+                           deck_options, NULL);
 
-  gnome_config_push_prefix (prefix);
-  gnome_config_set_string ("Variation/Default File", game_file);
-  gnome_config_set_string ("Deck/Options", deck_options);
-  gnome_config_pop_prefix ();
-  gnome_config_sync();
-
-#ifndef GNOME_SESSION_BUG
-  argv[0] = "rm";
-  argv[1] = gnome_config_get_real_path (prefix);
-  gnome_client_set_discard_command (client, 2, argv);
-#endif  
   return TRUE;
 }
 
@@ -256,16 +254,25 @@ void new_game (gchar* file, guint *seedp )
 
   size = gh_call0(game_data->start_game_lambda);
   gh_eval_str ("(start-game)");
+
   min_w = gh_scm2int (gh_car (size))*get_horiz_offset() + 2*get_horiz_start();
   min_h = gh_scm2int (gh_cadr (size))*get_vert_offset() + 2*get_vert_start();
-
   gtk_widget_set_size_request (playing_area, min_w, min_h);
-  if (surface)
-    refresh_screen();
 
-  game_over = FALSE;
-  make_title();
-  end_of_game_test();
+  /* It is possible for some games to not have any moves right from the
+   * start. If this happens we redeal. */
+  if (!gh_scm2bool (gh_call0 (game_data->game_over_lambda))) {
+    new_game (file, seedp);
+  } else {
+    if (surface)
+      refresh_screen();
+    
+    undo_set_sensitive (FALSE);
+    redo_set_sensitive (FALSE);
+    
+    game_over = FALSE;
+    make_title();
+  }
 }
 
 GtkWidget *score_value;
@@ -289,7 +296,7 @@ GtkWidget *time_value;
  *}
  */
 
-gint timer_cb ()
+static gint timer_cb ()
 {
 #if 0
 /*  game_seconds++;
@@ -337,7 +344,7 @@ void timer_stop ()
  */
 
 
-void create_sol_board ()
+static void create_sol_board ()
 {
   playing_area = gtk_drawing_area_new ();
   gtk_widget_set_events (playing_area, 
@@ -363,55 +370,22 @@ void create_sol_board ()
 
 gboolean quit_app (GtkMenuItem *menuitem)
 {
-  GtkWidget *box;
-  gint response;
-
-  if (!game_over) {
-    box = gtk_message_dialog_new (GTK_WINDOW (app),
-            GTK_DIALOG_MODAL,
-            GTK_MESSAGE_QUESTION,
-            GTK_BUTTONS_NONE,
-            _("Are you sure you want to quit Aisleriot?"));
-    gtk_dialog_add_buttons (GTK_DIALOG (box),
-            GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
-            GTK_STOCK_QUIT, GTK_RESPONSE_ACCEPT,
-            NULL);
-
-    
-    gtk_dialog_set_default_response (GTK_DIALOG (box), GTK_RESPONSE_REJECT);
-
-    response = gtk_dialog_run (GTK_DIALOG (box));
-    gtk_widget_destroy (box);
-
-    if (response == GTK_RESPONSE_REJECT)
-        return TRUE;
-  }
-  
-  gtk_widget_destroy (app);
   gtk_main_quit ();
 }
 
-void create_main_window ()
+static void create_main_window ()
 {
-  /* This is the prefix used to retrieve the state when NOT restarted: */
-  const gchar* prefix = 
-    gnome_client_get_global_config_prefix (gnome_master_client ());
-
-  app = gnome_app_new ("sol", _("Aisleriot"));
-  /* Use "prefix" as the default config location ... */
-  gnome_config_push_prefix (prefix);
-  /* ... and use it for the menubar and toolbar aw well: */
-  GNOME_APP (app)->prefix = (gchar*)prefix;
+  app = gnome_app_new ("aisleriot", _("Aisleriot"));
 
   gtk_widget_realize (app);
-
+  
   g_signal_connect (GTK_OBJECT (app), "delete_event", 
 		      GTK_SIGNAL_FUNC (quit_app), NULL);
   g_signal_connect (GTK_OBJECT (app), "configure_event",
 		      GTK_SIGNAL_FUNC (configure_event), NULL);
 }
 
-void create_press_data ()
+static void create_press_data ()
 {
   GdkWindowAttr attributes;
 
@@ -431,10 +405,10 @@ void create_press_data ()
 
 gchar* start_game;
 
-void main_prog(int argc, char *argv[])
+static void main_prog(int argc, char *argv[])
 {
   GtkWidget *score_label, *time_label, *score_box, *status_bar;
-
+  
   seed = time(NULL);
   g_random_set_seed(seed);
 
@@ -452,11 +426,11 @@ void main_prog(int argc, char *argv[])
   create_menus ();
 
   score_box = gtk_hbox_new(0, FALSE);
-  score_label = gtk_label_new (_("Score: "));
+  score_label = gtk_label_new (_("Score:"));
   gtk_box_pack_start (GTK_BOX(score_box), score_label, FALSE, FALSE, 0);
   score_value = gtk_label_new ("   0");
   gtk_box_pack_start (GTK_BOX(score_box), score_value, FALSE, FALSE, 0);
-  time_label = gtk_label_new (_("Time: "));
+  time_label = gtk_label_new (_("Time:"));
   gtk_box_pack_start (GTK_BOX(score_box), time_label, FALSE, FALSE, 0);
   time_value = games_clock_new ();
   gtk_box_pack_start (GTK_BOX(score_box), time_value, FALSE, FALSE, GNOME_PAD_SMALL);
@@ -478,6 +452,11 @@ void main_prog(int argc, char *argv[])
   splash_destroy ();
 
   gtk_widget_show (app);
+
+  if (!gconf_client_get_bool (gconf_client,
+                             "/apps/aisleriot/show_toolbar", NULL))
+    toolbar_hide();
+
   create_press_data ();
 
   gtk_widget_pop_colormap ();
@@ -485,31 +464,22 @@ void main_prog(int argc, char *argv[])
   gtk_main ();
 
   free_pixmaps();
-  gdk_drawable_unref (surface);
-  gdk_drawable_unref (press_data->moving_cards);
+  g_object_unref (surface);
+  g_object_unref (press_data->moving_cards);
 }
 
 static void
 retrieve_state (GnomeClient *client)
 {
-  const gchar *prefix = gnome_client_get_config_prefix (client);
-
-  gnome_config_push_prefix (prefix);
-  start_game = gnome_config_get_string_with_default 
-    ("Variation/Default File=" DEFAULT_VARIATION, NULL);
-  deck_options = gnome_config_get_string ("Deck/Options");
-  gnome_config_pop_prefix ();
-
-#ifdef GNOME_SESSION_BUG
-  if (strcmp (prefix, gnome_client_get_global_config_prefix (client))) {
-    gchar *prefix = gnome_config_get_real_path (prefix);
-    prefix[ strlen (prefix) - 1] = '\0';
-    unlink (prefix);
-  }
-#endif  
+  start_game = games_gconf_get_string (gconf_client,
+                                       "/apps/aisleriot/game_file",
+                                       "klondike.scm");
+  deck_options = games_gconf_get_string (gconf_client,
+                                         "/apps/aisleriot/card_options",
+                                         "beige.png bonded.png gnome.png bold-09x14.png knuth-09x10.png knuth-18x21.png knuth-21x25.png");
 }
 
-int
+static int
 is_game (const struct dirent* dent)
 {
   return (!strcmp (g_extension_pointer (dent->d_name),"scm") &&
@@ -530,18 +500,19 @@ int main (int argc, char *argv [])
   aisleriot_opts[0].descrip = N_("Variation on game rules");
   aisleriot_opts[0].argDescrip = N_("NAME");
 
-  gnome_score_init ("Aisleriot");
-
   bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
-   gnome_program_init ("aisleriot", VERSION,
+  gnome_program_init ("aisleriot", VERSION,
  		      LIBGNOMEUI_MODULE, 
  		      argc, argv,
  		      GNOME_PARAM_POPT_TABLE, aisleriot_opts,
  		      GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
 
+  gconf_client = gconf_client_get_default ();
+  games_gconf_sanity_check_string (gconf_client, "/apps/aisleriot/game_file");
+  
   gtk_widget_push_colormap (gdk_rgb_get_colormap ());
 
   gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-aisleriot.png");
@@ -551,7 +522,7 @@ int main (int argc, char *argv [])
 
   g_signal_connect (GTK_OBJECT (gnome_master_client ()), "save_yourself",
 		      GTK_SIGNAL_FUNC (save_state), 
-		      (gpointer) g_basename(argv[0]));
+		      (gpointer) g_path_get_basename(argv[0]));
   g_signal_connect (GTK_OBJECT (gnome_master_client ()), "die",
 		      GTK_SIGNAL_FUNC (quit_app), NULL);
 
