@@ -40,7 +40,6 @@
 #include "card.h"
 #include "menu.h"
 #include "dialog.h"
-#include "splash.h"
 
 #include "game.h"
 #include "hand.h"
@@ -76,17 +75,27 @@ gboolean         events_pending = false;
 static GConfClient *gconf_client = NULL;
 
 gfloat wager_value = 5.0;
+gfloat balance_value = 0.0;
+gboolean quick_deals = FALSE;
+gboolean show_probabilities = FALSE;
+gboolean show_toolbar = TRUE;
+gchar *game_variation = NULL;
 
-#define DEFAULT_VARIATION "Vegas_Strip.rules"
-#define GNOME_SESSION_BUG
-
+#define DEFAULT_VARIATION      "Vegas_Strip.rules"
+#define KEY_DIR                "/apps/blackjack"
+#define KEY_GAME_VARIATION     "/apps/blackjack/settings/variation"
+#define KEY_BALANCE            "/apps/blackjack/global/balance"
+#define KEY_SHOW_PROBABILITIES "/apps/blackjack/settings/show_probabilities"
+#define KEY_QUICK_DEAL         "/apps/blackjack/settings/quick_deal"
+#define KEY_SHOW_TOOLBAR       "/apps/blackjack/toolbar"
+#define KEY_DECK_OPTIONS       "/apps/blackjack/deck/options"
 
 void
 bj_make_window_title (gchar *game_name, gint seed) 
 {
   char *title;
 
-  title = g_strdup_printf (_("Blackjack:  %s  ( %d )"), game_name, seed);
+  title = g_strdup_printf (_("Blackjack: %s (%d)"), game_name, seed);
 
   gtk_window_set_title (GTK_WINDOW (app), title); 
 
@@ -100,7 +109,6 @@ bj_gui_show_toolbar (gboolean do_toolbar)
 
   toolbar_gdi = gnome_app_get_dock_item_by_name (GNOME_APP(app), 
                                                  GNOME_APP_TOOLBAR_NAME);
-  
   if (do_toolbar) {
     gtk_widget_show (GTK_WIDGET(toolbar_gdi));
   }
@@ -196,30 +204,6 @@ bj_create_board ()
 gboolean
 bj_quit_app (GtkMenuItem *menuitem)
 {
-  GtkWidget *box;
-  gint response;
-
-  if (bj_game_is_active ())
-    {
-      box = gtk_message_dialog_new (GTK_WINDOW (app),
-                                    GTK_DIALOG_MODAL,
-                                    GTK_MESSAGE_QUESTION,
-                                    GTK_BUTTONS_NONE,
-                                    _("Are you sure you want to quit Blackjack?"));
-      gtk_dialog_add_buttons (GTK_DIALOG (box),
-                              GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
-                              GTK_STOCK_QUIT, GTK_RESPONSE_ACCEPT,
-                              NULL);
-    
-      gtk_dialog_set_default_response (GTK_DIALOG (box), GTK_RESPONSE_REJECT);
-      
-      response = gtk_dialog_run (GTK_DIALOG (box));
-      gtk_widget_destroy (box);
-
-      if (response == GTK_RESPONSE_REJECT)
-        return TRUE;
-    }
-
   delete rules;
   delete strategy;
   delete dealer;
@@ -301,8 +285,6 @@ create_chip_stack_press_data ()
   chip_stack_press_data->status = 0;
 }
 
-gchar* start_game;
-
 static void
 main_prog(int argc, char *argv[])
 {
@@ -311,15 +293,11 @@ main_prog(int argc, char *argv[])
   seed = time (NULL);
   g_random_set_seed (seed);
 
-  splash_update (_("Loading images..."), 0.20);
-
   create_main_window ();
 
   bj_card_load_pixmaps (app, bj_get_deck_options ());
   bj_slot_load_pixmaps ();
  
-  splash_update (_("Computing basic strategy..."), 0.50);
-
   bj_create_board ();
   bj_menu_create ();
 
@@ -356,7 +334,7 @@ main_prog(int argc, char *argv[])
   balance_label = gtk_label_new (_("Balance:"));
   gtk_box_pack_start (GTK_BOX (group_box), balance_label, 
                       FALSE, FALSE, 0);
-  balance_value_label = gtk_label_new ("0.00");
+  balance_value_label = gtk_label_new (g_strdup_printf ("%.2f", balance_value));
   gtk_box_pack_start (GTK_BOX (group_box), balance_value_label, 
                       FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (balance_box), group_box, FALSE, FALSE, 0);
@@ -365,12 +343,12 @@ main_prog(int argc, char *argv[])
 
   bj_menu_install_hints (GNOME_APP (app));
 
-  bj_game_new (start_game, &seed);
-
-  splash_destroy ();
+  bj_game_new (bj_get_game_variation (), &seed);
 
   gtk_widget_show (app);
 
+  bj_gui_show_toolbar (show_toolbar);
+
   create_press_data ();
   create_chip_stack_press_data ();
 
@@ -379,43 +357,6 @@ main_prog(int argc, char *argv[])
   gtk_main ();
 }
 
-gchar *
-bj_gconf_client_get_string_or_set_default (GConfClient *client, const gchar* key, const gchar* def)
-{
-  GError *error = NULL;
-  gchar *value;
-  gboolean use_default = false;
-
-  g_return_val_if_fail (client != NULL, NULL);
-  g_return_val_if_fail (key != NULL, NULL);
-  
-  value = gconf_client_get_string (client, key, &error);
-
-  if (error != NULL)
-    {
-      g_warning (error->message);
-      /* If a default was passed return that, otherwise NULL */
-      use_default = true;
-    }
-
-  /* value is null if key was not set */
-  if (value == NULL)
-    {
-      /* If a default was passed return that, otherwise NULL */
-      use_default = true;
-    }
-
-  if (use_default)
-    {
-      if (def)
-        {
-          gconf_client_set_string (client, key, def, NULL);
-        }
-      return g_strdup (def);
-    }
-  return value;
-}
-
 GConfClient *
 bj_gconf_client ()
 {
@@ -428,10 +369,9 @@ static void
 bj_gconf_balance_cb (GConfClient *client, guint cnxn_id, 
                      GConfEntry *entry, gpointer user_data)
 {
-  gdouble balance;
-
-  balance = bj_get_balance ();
-  bj_show_balance (balance);
+  balance_value = gconf_client_get_float 
+    (client, KEY_BALANCE, NULL);
+  bj_show_balance (balance_value);
 }
 
 static void
@@ -450,6 +390,17 @@ static void
 bj_gconf_show_probabilities_cb (GConfClient *client, guint cnxn_id, 
                                 GConfEntry *entry, gpointer user_data)
 {
+  show_probabilities = gconf_client_get_bool 
+    (client, KEY_SHOW_PROBABILITIES, NULL);
+  bj_draw_refresh_screen ();
+}
+
+static void
+bj_gconf_quick_deal_cb (GConfClient *client, guint cnxn_id, 
+                        GConfEntry *entry, gpointer user_data)
+{
+  quick_deals = gconf_client_get_bool 
+    (client, KEY_QUICK_DEAL, NULL);
   bj_draw_refresh_screen ();
 }
 
@@ -459,7 +410,7 @@ bj_gconf_show_toolbar_cb (GConfClient *client, guint cnxn_id,
 {
   gboolean show_toolbar;
 
-  show_toolbar = bj_get_show_toolbar ();
+  show_toolbar = gconf_client_get_bool (client, KEY_SHOW_TOOLBAR, NULL);
   bj_gui_show_toolbar (show_toolbar);
 }
 
@@ -469,7 +420,7 @@ bj_get_deck_options ()
   GdkCardDeckOptions deck_options;
 
   deck_options = gconf_client_get_string (bj_gconf_client (), 
-                                          "/apps/blackjack/deck/options", 
+                                          KEY_DECK_OPTIONS,
                                           NULL);
   return deck_options;
 }
@@ -478,7 +429,7 @@ void
 bj_set_deck_options (GdkCardDeckOptions value)
 {
   gconf_client_set_string (bj_gconf_client (), 
-                           "/apps/blackjack/deck/options", 
+                           KEY_DECK_OPTIONS,
                            value,
                            NULL);
 }
@@ -486,79 +437,75 @@ bj_set_deck_options (GdkCardDeckOptions value)
 gboolean
 bj_get_show_probabilities ()
 {
-  return gconf_client_get_bool (bj_gconf_client (), 
-                                "/apps/blackjack/settings/show_probabilities", 
-                                NULL);
+  return show_probabilities;
 }
 void
 bj_set_show_probabilities (gboolean value)
 {
   gconf_client_set_bool (bj_gconf_client (), 
-                         "/apps/blackjack/settings/show_probabilities", 
+                         KEY_SHOW_PROBABILITIES,
                          value, NULL);
 }
 
 gboolean
 bj_get_show_toolbar ()
 {
-  return gconf_client_get_bool (bj_gconf_client (), 
-                                "/apps/blackjack/toolbar", 
-                                NULL);
+  return show_toolbar;
 }
 void
 bj_set_show_toolbar (gboolean value)
 {
+  show_toolbar = value;
+  bj_gui_show_toolbar (value);
   gconf_client_set_bool (bj_gconf_client (), 
-                         "/apps/blackjack/toolbar", 
+                         KEY_SHOW_TOOLBAR,
                          value, NULL);
 }
 
 gboolean
 bj_get_quick_deal ()
 {
-  return gconf_client_get_bool (bj_gconf_client (), 
-                                "/apps/blackjack/settings/quick_deal", 
-                                NULL);
+  return quick_deals;
 }
 void
 bj_set_quick_deal (gboolean value)
 {
   gconf_client_set_bool (bj_gconf_client (), 
-                         "/apps/blackjack/settings/quick_deal", 
+                         KEY_QUICK_DEAL,
                          value, NULL);
 }
 
 gchar *
 bj_get_game_variation ()
 {
-  char *value;
-  value = gconf_client_get_string (bj_gconf_client (), 
-                                   "/apps/blackjack/settings/variation", 
-                                   NULL);
-  return value;
+  return g_strdup (game_variation);
 }
 
 void
 bj_set_game_variation (const gchar *value)
 {
+  if (game_variation)
+    g_free (game_variation);
+  game_variation = g_strdup (value);
   gconf_client_set_string (bj_gconf_client (), 
-                           "/apps/blackjack/settings/variation", 
-                           value, NULL);
+                           KEY_GAME_VARIATION,
+                           game_variation, NULL);
 }
 
 gdouble
 bj_get_balance ()
 {
-  gdouble balance;
-
-  balance = gconf_client_get_float (bj_gconf_client (), "/apps/blackjack/global/balance", NULL);
-  return balance;
+  return balance_value;
 }
 
 void
 bj_set_balance (gdouble balance)
 {
-  gconf_client_set_float (bj_gconf_client (), "/apps/blackjack/global/balance", balance, NULL);
+  balance_value = balance;
+  bj_show_balance (balance_value);
+  gconf_client_set_float (bj_gconf_client (),
+                          KEY_BALANCE,
+                          balance, NULL);
 }
 
 void
@@ -575,26 +522,53 @@ bj_gconf_init (GConfClient *client)
 {
   GError *error = NULL;
   gdouble balance;
+  gchar *variation_tmp;
 
-  start_game = bj_gconf_client_get_string_or_set_default
-    (client, "/apps/blackjack/settings/variation", DEFAULT_VARIATION);
+  variation_tmp = gconf_client_get_string (client,
+                                           KEY_GAME_VARIATION,
+                                           NULL);
+  if (variation_tmp) 
+    {
+      game_variation = g_strdup (variation_tmp);
+      g_free (variation_tmp);
+    }
+  else
+    game_variation = g_strdup (DEFAULT_VARIATION);
 
-  balance = bj_get_balance ();
+  balance = gconf_client_get_float (client,
+                                    KEY_BALANCE,
+                                    NULL);
+  if (balance)
+    balance_value = balance;
+
+  show_probabilities = gconf_client_get_bool (client,
+                                              KEY_SHOW_PROBABILITIES,
+                                              NULL);
+  quick_deals = gconf_client_get_bool (client, 
+                                       KEY_QUICK_DEAL,
+                                       NULL);
+  show_toolbar = gconf_client_get_bool (client, 
+                                        KEY_SHOW_TOOLBAR,
+                                        NULL);
 
   gconf_client_notify_add (client,
-                           "/apps/blackjack/global/balance",
+                           KEY_BALANCE,
                            bj_gconf_balance_cb,
                            NULL, NULL, NULL);
   gconf_client_notify_add (client,
-                           "/apps/blackjack/deck/options",
+                           KEY_DECK_OPTIONS,
                            bj_gconf_deck_options_cb,
                            NULL, NULL, NULL);
   gconf_client_notify_add (client,
-                           "/apps/blackjack/settings/show_probabilities",
+                           KEY_SHOW_PROBABILITIES,
                            bj_gconf_show_probabilities_cb,
                            NULL, NULL, NULL);
   gconf_client_notify_add (client,
-                           "/apps/blackjack/toolbar",
+                           KEY_QUICK_DEAL,
+                           bj_gconf_quick_deal_cb,
+                           NULL, NULL, NULL);
+  gconf_client_notify_add (client,
+                           KEY_SHOW_TOOLBAR,
                            bj_gconf_show_toolbar_cb,
                            NULL, NULL, NULL);
 }
@@ -622,16 +596,15 @@ main (int argc, char *argv [])
                       GNOME_PARAM_POPT_TABLE, blackjack_opts,
                       GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
 
-  gconf_init(argc, argv, NULL);
-  gconf_client_add_dir (bj_gconf_client (), "/apps/blackjack",
+  gconf_init (argc, argv, NULL);
+  gconf_client_add_dir (bj_gconf_client (), KEY_DIR,
                         GCONF_CLIENT_PRELOAD_NONE, NULL);
 
   gtk_widget_push_colormap (gdk_rgb_get_colormap ());
 
-  gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-blackjack.png");
-  splash_new ();
-
-  splash_update (_("Initializing..."), 0.10);
+  gchar *icon_path = g_build_filename (GNOME_ICONDIR, "gnome-blackjack.png", NULL);
+  gnome_window_icon_set_default_from_file (icon_path);
+  g_free (icon_path);
 
   g_signal_connect (GTK_OBJECT (gnome_master_client ()), "die",
                     GTK_SIGNAL_FUNC (bj_quit_app), NULL);
