@@ -1,3 +1,5 @@
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
+
 /*
  * properties.c - properties for gataxx
  * Written by Chris Rogers (gandalf@pobox.com)
@@ -22,8 +24,10 @@
 
 #include "config.h"
 #include <gnome.h>
+#include <string.h>
 #include <dirent.h>
 #include <games-clock.h>
+#include <games-frame.h>
 #include <gconf/gconf-client.h>
 
 #include "properties.h"
@@ -57,7 +61,8 @@ gchar * gataxx_gconf_get_string (gchar *key);
 gint gataxx_gconf_get_int (gchar *key, gint default_int);
 gboolean gataxx_gconf_get_bool (gchar *key, gint default_bool);
 
-void load_properties ()
+void 
+load_properties (void)
 {
 	black_computer_level =
 		gataxx_gconf_get_int ("/apps/gataxx/blacklevel", 0);
@@ -93,7 +98,8 @@ void load_properties ()
 	}
 }
 
-void save_properties ()
+static void
+save_properties (void)
 {
 	gconf_client_set_int (gataxx_gconf_client, "/apps/gataxx/blacklevel",
 	                      black_computer_level, NULL);
@@ -109,7 +115,8 @@ void save_properties ()
 	                       flip_final, NULL);
 }
 
-void apply_changes ()
+static void
+apply_changes (void)
 {
 	guint i, j;
 
@@ -172,27 +179,30 @@ void apply_changes ()
 	check_computer_players ();
 }
 
-void black_computer_level_select (GtkWidget *widget, gpointer data)
+static void
+black_computer_level_select (GtkWidget *widget, gpointer data)
 {
-	if (((guint) data != black_computer_level) &&
-		       (GTK_TOGGLE_BUTTON (widget)->active)) {
+	if (((guint) data != black_computer_level) 
+	    && (GTK_TOGGLE_BUTTON (widget)->active)) {
 		black_computer_level = (guint) data;
 	}
 	save_properties ();
 	apply_changes ();
 }
 
-void white_computer_level_select (GtkWidget *widget, gpointer data)
+static void
+white_computer_level_select (GtkWidget *widget, gpointer data)
 {
-	if (((guint) data != white_computer_level) &&
-		       (GTK_TOGGLE_BUTTON (widget)->active)) {
+	if (((guint) data != white_computer_level) 
+	    && (GTK_TOGGLE_BUTTON (widget)->active)) {
 		white_computer_level = (guint) data;
 	}
 	save_properties ();
 	apply_changes ();
 }
 
-void quick_moves_select (GtkWidget *widget, gpointer data)
+static void
+quick_moves_select (GtkWidget *widget, gpointer data)
 {
 	if (GTK_TOGGLE_BUTTON (widget)->active) {
 		quick_moves = TRUE;
@@ -205,7 +215,8 @@ void quick_moves_select (GtkWidget *widget, gpointer data)
 	apply_changes ();
 }
 
-void flip_final_select (GtkWidget *widget, gpointer data)
+static void
+flip_final_select (GtkWidget *widget, gpointer data)
 {
 	if (GTK_TOGGLE_BUTTON (widget)->active) {
 		flip_final = TRUE;
@@ -218,7 +229,8 @@ void flip_final_select (GtkWidget *widget, gpointer data)
 }
 
 
-void animate_select (GtkWidget *widget, gpointer data)
+static void
+animate_select (GtkWidget *widget, gpointer data)
 {
 	if (GTK_TOGGLE_BUTTON (widget)->active) {
 		animate = (gint) data;
@@ -228,18 +240,8 @@ void animate_select (GtkWidget *widget, gpointer data)
 	apply_changes ();
 }
 
-void apply_cb (GtkWidget *widget, gpointer data)
-{
-	save_properties ();
-	apply_changes();
-}
-
-void destroy_cb (GtkWidget *widget, gpointer data)
-{
-	mapped = 0;
-}
-
-void set_selection(GtkWidget *widget, gpointer data)
+void
+set_selection (GtkWidget *widget, gpointer data)
 {
 	if (tile_set)
 		g_free (tile_set);
@@ -250,50 +252,55 @@ void set_selection(GtkWidget *widget, gpointer data)
 	apply_changes();
 }
 
-void free_str(GtkWidget *widget, void *data)
+void
+free_str (GtkWidget *widget, void *data)
 {
         free(data);
 }
 
-void fill_menu(GtkWidget *menu)
+void
+fill_menu (GtkWidget *menu)
 {
         struct dirent *e;
-	char *dname = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP, ("gataxx"), FALSE, NULL);
+	gchar *dname = gnome_program_locate_file (NULL,
+						  GNOME_FILE_DOMAIN_APP_PIXMAP,
+						  ("gataxx"), FALSE, NULL);
         DIR *dir;
         int itemno = 0;
 
-        dir = opendir(dname);
+        dir = opendir (dname);
 
-        if(!dir) {
+        if (! dir) {
                 return;
 	}
 
-        while((e = readdir(dir)) != NULL) {
+        while ((e = readdir (dir)) != NULL) {
                 GtkWidget *item;
-                char *s = GINT_TO_POINTER (g_strdup(e->d_name));
-                if(!strstr(e->d_name, ".png")) {
-                        free(s);
+                char *s = GINT_TO_POINTER (g_strdup (e->d_name));
+                if(! g_strrstr (e->d_name, ".png")) {
+                        g_free (s);
                         continue;
                 }
 
-                item = gtk_menu_item_new_with_label(s);
-                gtk_widget_show(item);
+                item = gtk_menu_item_new_with_label (s);
+                gtk_widget_show (item);
                 gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-                g_signal_connect(G_OBJECT(item), "activate",
-				 G_CALLBACK (set_selection), s);
-                g_signal_connect(G_OBJECT(item), "destroy",
-				 G_CALLBACK (free_str), s);
+                g_signal_connect (G_OBJECT (item), "activate",
+				  G_CALLBACK (set_selection), s);
+                g_signal_connect (G_OBJECT (item), "destroy",
+				  G_CALLBACK (free_str), s);
 
-                if (!strcmp(tile_set, s)) {
-                        gtk_menu_set_active(GTK_MENU(menu), itemno);
+                if (! strcmp (tile_set, s)) {
+                        gtk_menu_set_active (GTK_MENU (menu), itemno);
                 }
 
                 itemno++;
         }
-        closedir(dir);
+        closedir (dir);
 }
 
-void show_properties_dialog ()
+void
+show_properties_dialog (void)
 {
 	GtkWidget *notebook;
 	GtkWidget *frame;
@@ -302,7 +309,7 @@ void show_properties_dialog ()
 	GtkWidget *table;
 	GtkWidget *button;
 	GtkWidget *menu;
-	GtkWidget *vbox;
+	GtkWidget *vbox, *vbox2;
 	GtkWidget *hbox;
 	GtkWidget *option_menu;
 	
@@ -311,7 +318,7 @@ void show_properties_dialog ()
 		return;
 	}
 
-        propbox = gtk_dialog_new_with_buttons (_("Properties"),
+        propbox = gtk_dialog_new_with_buttons (_("Gataxx Preferences"),
 					       GTK_WINDOW (window),
 					       GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
 					       GTK_STOCK_CLOSE,
@@ -331,29 +338,29 @@ void show_properties_dialog ()
 	/* Players tab */
 	
 	label = gtk_label_new (_("Players"));
-	gtk_widget_show (label);
 
-	table = gtk_table_new (2, 2, FALSE);
+	vbox = gtk_vbox_new (FALSE, 0);
+	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);
+
+	table = gtk_table_new (1, 2, FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (table), 6);
-	gtk_widget_show (table);
-	
+        gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
+
+	vbox2 = gtk_vbox_new (FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 12);
+        gtk_box_pack_start (GTK_BOX (vbox), vbox2, FALSE, FALSE, 0);
 	button = gtk_check_button_new_with_label (_("Quick Moves (cut computer delay in half)"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				     (computer_speed == COMPUTER_MOVE_DELAY / 2));
 	g_signal_connect (G_OBJECT (button), "toggled",
 			  G_CALLBACK (quick_moves_select), NULL);
-	gtk_widget_show (button);
+	gtk_box_pack_start (GTK_BOX (vbox2), button, FALSE, FALSE, 0);
 	
-	gtk_table_attach (GTK_TABLE (table), button, 0, 2, 1, 2,
-			  GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL,
-			  6, 6);
 	
-	frame = gtk_frame_new (_("Dark"));
-	gtk_widget_show (frame);
+	frame = games_frame_new (_("Dark"));
 	
-	vbox = gtk_vbox_new (TRUE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
-	gtk_widget_show (vbox);
+	vbox2 = gtk_vbox_new (FALSE, 6);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 6);
 	
 	button = gtk_radio_button_new_with_label (NULL, _("Human"));
 	if (black_computer_level == 0) {
@@ -361,9 +368,8 @@ void show_properties_dialog ()
 	}
 	g_signal_connect (G_OBJECT (button), "toggled",
 			  G_CALLBACK (black_computer_level_select), (gpointer) 0);
-	gtk_widget_show (button);
 
-	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox2), button, FALSE, FALSE, 0);
 
 	button = gtk_radio_button_new_with_label (gtk_radio_button_get_group
 		      				 (GTK_RADIO_BUTTON (button)), _("Level one"));
@@ -372,9 +378,8 @@ void show_properties_dialog ()
 	}
 	g_signal_connect (G_OBJECT (button), "toggled",
 			  G_CALLBACK (black_computer_level_select), (gpointer) 1);
-	gtk_widget_show (button);
 	
-	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox2), button, FALSE, FALSE, 0);
 	button = gtk_radio_button_new_with_label (gtk_radio_button_get_group
 						 (GTK_RADIO_BUTTON (button)), _("Level two"));
 	if (black_computer_level == 2) {
@@ -382,9 +387,8 @@ void show_properties_dialog ()
 	}
 	g_signal_connect (G_OBJECT (button), "toggled",
 			  G_CALLBACK (black_computer_level_select), (gpointer) 2);
-	gtk_widget_show (button);
 
-	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox2), button, FALSE, FALSE, 0);
 	button = gtk_radio_button_new_with_label (gtk_radio_button_get_group
 			                         (GTK_RADIO_BUTTON (button)), _("Level three"));
 	if (black_computer_level == 3) {
@@ -392,22 +396,19 @@ void show_properties_dialog ()
 	}
 	g_signal_connect (G_OBJECT (button), "toggled",
 			  G_CALLBACK (black_computer_level_select), (gpointer) 3);
-	gtk_widget_show (button);
 
-	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox2), button, FALSE, FALSE, 0);
 
-	gtk_container_add (GTK_CONTAINER (frame), vbox);
+	gtk_container_add (GTK_CONTAINER (frame), vbox2);
 	
 	gtk_table_attach (GTK_TABLE (table), frame, 0, 1, 0, 1,
 			  GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL,
 			  6, 6);
 	
-	frame = gtk_frame_new (_("Light"));
-	gtk_widget_show (frame);
+	frame = games_frame_new (_("Light"));
 	
-	vbox = gtk_vbox_new (TRUE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
-	gtk_widget_show (vbox);
+	vbox2 = gtk_vbox_new (FALSE, 6);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 6);
 	
 	button = gtk_radio_button_new_with_label (NULL, _("Human"));
 	if (white_computer_level == 0) {
@@ -415,8 +416,7 @@ void show_properties_dialog ()
 	}
 	g_signal_connect (G_OBJECT (button), "toggled",
 			  G_CALLBACK (white_computer_level_select), (gpointer) 0);
-	gtk_widget_show (button);
-	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox2), button, FALSE, FALSE, 0);
 	button = gtk_radio_button_new_with_label (gtk_radio_button_get_group
 			                         (GTK_RADIO_BUTTON (button)), _("Level one"));
 	if (white_computer_level == 1) {
@@ -424,8 +424,7 @@ void show_properties_dialog ()
 	}
 	g_signal_connect (G_OBJECT (button), "toggled",
 			  G_CALLBACK (white_computer_level_select), (gpointer) 1);
-	gtk_widget_show (button);
-	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox2), button, FALSE, FALSE, 0);
 	button = gtk_radio_button_new_with_label (gtk_radio_button_get_group
 			                         (GTK_RADIO_BUTTON (button)), _("Level two"));
 	if (white_computer_level == 2) {
@@ -433,8 +432,7 @@ void show_properties_dialog ()
 	}
 	g_signal_connect (G_OBJECT (button), "toggled",
 			  G_CALLBACK (white_computer_level_select), (gpointer) 2);
-	gtk_widget_show (button);
-	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox2), button, FALSE, FALSE, 0);
 	button = gtk_radio_button_new_with_label (gtk_radio_button_get_group
 			                         (GTK_RADIO_BUTTON (button)), _("Level three"));
 	if (white_computer_level == 3) {
@@ -442,32 +440,25 @@ void show_properties_dialog ()
 	}
 	g_signal_connect (G_OBJECT (button), "toggled",
 			  G_CALLBACK (white_computer_level_select), (gpointer) 3);
-	gtk_widget_show (button);
-	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox2), button, FALSE, FALSE, 0);
 	
-	gtk_container_add (GTK_CONTAINER (frame), (vbox));
+	gtk_container_add (GTK_CONTAINER (frame), (vbox2));
 
 	gtk_table_attach (GTK_TABLE (table), frame, 1, 2, 0, 1,
 			  GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL,
 			  6, 6);
 
-	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), table, label);
-
-	/* Animation tab */
+	/* Appearance tab */
 	
-	label = gtk_label_new (_("Animation"));
-	gtk_widget_show (label);
+	label = gtk_label_new (_("Appearance"));
 
 	table = gtk_table_new (1, 2, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (table), GNOME_PAD_SMALL);
-	gtk_widget_show (table);
+	gtk_container_set_border_width (GTK_CONTAINER (table), 6);
 	
-	frame = gtk_frame_new ("Animation");
-	gtk_widget_show (frame);
+	frame = games_frame_new ("Animation");
 	
-	vbox = gtk_vbox_new (TRUE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD_SMALL);
-	gtk_widget_show (vbox);
+	vbox = gtk_vbox_new (FALSE, 6);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
 	
 	button = gtk_radio_button_new_with_label (NULL, _("None"));
 
@@ -476,7 +467,6 @@ void show_properties_dialog ()
 	}
 	g_signal_connect (G_OBJECT (button), "toggled",
 			  G_CALLBACK (animate_select), (gpointer) 0);
-	gtk_widget_show (button);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 	button = gtk_radio_button_new_with_label (gtk_radio_button_get_group
@@ -486,7 +476,6 @@ void show_properties_dialog ()
 	}
 	g_signal_connect (G_OBJECT (button), "toggled",
 			  G_CALLBACK (animate_select), (gpointer) 1);
-	gtk_widget_show (button);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 	button = gtk_radio_button_new_with_label (gtk_radio_button_get_group
@@ -496,31 +485,27 @@ void show_properties_dialog ()
 	}
 	g_signal_connect (G_OBJECT (button), "toggled",
 			  G_CALLBACK (animate_select), (gpointer) 2);
-	gtk_widget_show (button);
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 	
 	gtk_container_add (GTK_CONTAINER (frame), vbox);
 
-	gtk_table_attach (GTK_TABLE (table), frame, 0, 1, 0, 1,
-			  GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL,
-			  6, 6);
-	
-	vbox = gtk_vbox_new (TRUE, 0);
-	gtk_widget_show (vbox);
+	gtk_table_attach_defaults (GTK_TABLE (table), frame, 0, 1, 0, 1);
+
+        frame = games_frame_new (_("Options"));
+	vbox = gtk_vbox_new (FALSE, 6);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
+	gtk_container_add (GTK_CONTAINER (frame), vbox);
 	
 	button = gtk_check_button_new_with_label (_("Flip final results"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), flip_final);
 	g_signal_connect (G_OBJECT (button), "toggled",
 			  G_CALLBACK (flip_final_select), NULL);
-	gtk_widget_show (button);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 	
 	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
-	gtk_widget_show (hbox);
 	
 	label2 = gtk_label_new (_("Tile set:"));
-	gtk_widget_show (label2);
 	
 	gtk_box_pack_start (GTK_BOX (hbox), label2, FALSE, FALSE, 0);
 	
@@ -528,19 +513,16 @@ void show_properties_dialog ()
 	menu = gtk_menu_new ();
 	fill_menu (menu);
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
-	gtk_widget_show (option_menu);
 	
 	gtk_box_pack_start (GTK_BOX (hbox), option_menu, TRUE, TRUE, 0);
 	
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 	
-	gtk_table_attach (GTK_TABLE (table), vbox, 1, 2, 0, 1,
-			  GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL,
-			  6, 6);
+	gtk_table_attach_defaults (GTK_TABLE (table), frame, 1, 2, 0, 1);
+
 	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), table, label);
 	
-	gtk_widget_show (notebook);
-	gtk_widget_show (propbox);
+	gtk_widget_show_all (propbox);
 
 }	
 
@@ -640,7 +622,8 @@ gataxx_gconf_get_string (gchar *key)
 	return retval;
 }
 
-void reload_properties ()
+void
+reload_properties (void)
 {
 	g_free (tile_set);
 	tile_set = NULL;
