@@ -31,6 +31,7 @@
 
 #define NUMWORMS 4
 
+#define NUM_COLORS 7
 #define WORMRED 12
 #define WORMGREEN 13
 #define WORMBLUE 14
@@ -56,15 +57,14 @@ void gnibbles_draw_big_pixmap (gint which, gint x, gint y);
 void gnibbles_draw_pixmap_buffer (gint which, gint x, gint y);
 void gnibbles_draw_big_pixmap_buffer (gint which, gint x, gint y);
 void gnibbles_load_pixmap (GtkWidget *window);
-void gnibbles_load_level (GtkWidget *window, int level);
-void gnibbles_init ();
-void gnibbles_add_bonus (int regular);
-void gnibbles_destroy ();
-gint gnibbles_move_worms ();
+void gnibbles_load_level (GtkWidget *window, gint level);
+void gnibbles_init (void);
+void gnibbles_add_bonus (gint regular);
+gint gnibbles_move_worms (void);
 gint gnibbles_keypress_worms (guint keyval);
 void gnibbles_undraw_worms (gint data);
 void gnibbles_play_sound (const char *which);
-void gnibbles_show_scores (gint pos);
-void gnibbles_log_score ();
+void gnibbles_show_scores (GtkWidget *window, gint pos);
+void gnibbles_log_score (GtkWidget *window);
 
 #endif
