@@ -1,3 +1,5 @@
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
+
 /* 
  *   Gnome Nibbles: Gnome Worm Game
  *   Written by Sean MacIsaac <sjm@acm.org>, Ian Peters <itp@gnu.org>
@@ -19,10 +21,13 @@
 
 #include <config.h>
 #include <gnome.h>
+#include <string.h>
 #include <libgnomeui/gnome-window-icon.h>
 #include <gdk/gdkkeysyms.h>
 #include <time.h>
+#include <gconf/gconf-client.h>
 
+#include "main.h"
 #include "properties.h"
 #include "gnibbles.h"
 #include "worm.h"
@@ -69,7 +74,7 @@ static struct _pointers {
 
 static gint main_loop (gpointer data);
 static gint add_bonus_cb (gpointer data);
-static void render_logo ();
+static void render_logo (void);
 static gint new_game_cb (GtkWidget *widget, gpointer data);
 gint pause_game_cb (GtkWidget *widget, gpointer data);
 static gint end_game_cb (GtkWidget *widget, gpointer data);
@@ -106,7 +111,8 @@ static GnomeUIInfo main_menu[] = {
 	GNOMEUIINFO_END
 };
 
-void hide_cursor ()
+static void
+hide_cursor (void)
 {
 	if (pointers.current != pointers.invisible) {
 		gdk_window_set_cursor (drawing_area->window, pointers.invisible);
@@ -114,7 +120,8 @@ void hide_cursor ()
 	}
 }
 
-void show_cursor ()
+static void
+show_cursor (void)
 {
 	if (pointers.current != NULL) {
 		gdk_window_set_cursor (drawing_area->window, NULL);
@@ -122,14 +129,16 @@ void show_cursor ()
 	}
 }
 
-gint game_running ()
+gint 
+game_running (void)
 {
 	return (main_id || erase_id || dummy_id || restart_id || paused);
 }
 
-static void zero_board ()
+static void
+zero_board (void)
 {
-	int i, j;
+	gint i, j;
 
 	for (i = 0; i < BOARDWIDTH; i++)
 		for (j = 0; j < BOARDHEIGHT; j++) {
@@ -138,55 +147,14 @@ static void zero_board ()
 		}
 }
 
-static gint end_game_box (short quit)
+static void
+quit_cb (GtkWidget *widget, gpointer data)
 {
-	gint pause_state;
-	static GtkWidget *box;
-	gint status;
-
-	if (box)
-		return 0;
-
-	pause_state = paused;
-	if (!paused)
-		pause_game_cb (NULL, (gpointer) 0);
-
-	if (!quit)
-		box = gtk_message_dialog_new (GTK_WINDOW (window),
-			       GTK_DIALOG_MODAL,
-			       GTK_MESSAGE_QUESTION,
-			       GTK_BUTTONS_YES_NO,
-			       _("Are you sure you want to start a new game?"));
-	else {
-		box = gtk_message_dialog_new (GTK_WINDOW (window),
-				GTK_DIALOG_MODAL,
-				GTK_MESSAGE_QUESTION,
-				GTK_BUTTONS_NONE,
-				_("Are you sure you want to quit Gnibbles?"));
-		gtk_dialog_add_buttons (GTK_DIALOG (box),
-				GTK_STOCK_CANCEL, GTK_RESPONSE_NO,
-				GTK_STOCK_QUIT, GTK_RESPONSE_YES,
-				NULL);
-	}
-	status = gtk_dialog_run (GTK_DIALOG (box)) == GTK_RESPONSE_NO;
-	gtk_widget_destroy (GTK_WIDGET (box));
-	box = NULL;
-	if (!pause_state && status)
-		pause_game_cb (NULL, (gpointer) 0);
-	return (status);
-
-}
-
-static void quit_cb (GtkWidget *widget, gpointer data)
-{
-	if (game_running () && end_game_box (1)) // 1 for quit
-		return;
-
-	gnibbles_destroy ();
 	gtk_main_quit ();
 }
 
-static void about_cb (GtkWidget *widget, gpointer data)
+static void
+about_cb (GtkWidget *widget, gpointer data)
 {
 	static GtkWidget *about = NULL;
 	GdkPixbuf *pixbuf = NULL;
@@ -217,13 +185,13 @@ static void about_cb (GtkWidget *widget, gpointer data)
 	}
 
 	about = gnome_about_new (_("Gnibbles"), VERSION,
-			"(C) 1999 Sean MacIsaac and Ian Peters",
-			_("Send comments and bug reports to: "
-			  "sjm@acm.org, itp@gnu.org"),
-			(const char **)authors,
-			(const char **)documenters,
-			strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
-			pixbuf);
+				 "Copyright \xc2\xa9 1999-2003 Sean MacIsaac, "
+				 "Ian Peters",
+				 _("A worm game for GNOME."),
+				 (const char **)authors,
+				 (const char **)documenters,
+				 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
+				 pixbuf);
    
 	if (pixbuf != NULL)
 		gdk_pixbuf_unref (pixbuf);
@@ -236,10 +204,11 @@ static void about_cb (GtkWidget *widget, gpointer data)
 
 static gint expose_event_cb (GtkWidget *widget, GdkEventExpose *event)
 {
-	gdk_draw_drawable (GDK_DRAWABLE (widget->window), widget->style->fg_gc[GTK_WIDGET_STATE
-			(widget)], buffer_pixmap, event->area.x, event->area.y,
-			event->area.x, event->area.y, event->area.width,
-			event->area.height);
+	gdk_draw_drawable (GDK_DRAWABLE (widget->window),
+			   widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
+			   buffer_pixmap, event->area.x, event->area.y,
+			   event->area.x, event->area.y, event->area.width,
+			   event->area.height);
 
 	return (FALSE);
 }
@@ -250,7 +219,8 @@ static gint key_press_cb (GtkWidget *widget, GdkEventKey *event)
 	return gnibbles_keypress_worms (event->keyval);
 }
 
-static void draw_board ()
+static void
+draw_board ()
 {
 	int i, j;
 
@@ -277,13 +247,15 @@ static void draw_board ()
 		gnibbles_bonus_draw (boni->bonuses[i]);
 
 	gdk_draw_drawable (GDK_DRAWABLE (drawing_area->window),
-			 drawing_area->style->fg_gc[GTK_WIDGET_STATE
-			 (drawing_area)], buffer_pixmap, 0, 0, 0, 0,
-			 BOARDWIDTH*properties->tilesize, BOARDHEIGHT*properties->tilesize);
+			   drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)],
+			   buffer_pixmap, 0, 0, 0, 0,
+			   BOARDWIDTH*properties->tilesize,
+			   BOARDHEIGHT*properties->tilesize);
 	
 }
 
-static gint configure_event_cb (GtkWidget *widget, GdkEventConfigure *event)
+static gint
+configure_event_cb (GtkWidget *widget, GdkEventConfigure *event)
 {
 	gnibbles_load_pixmap (window);
 	if (buffer_pixmap)
@@ -300,21 +272,24 @@ static gint configure_event_cb (GtkWidget *widget, GdkEventConfigure *event)
 	return (FALSE);
 }
 
-static gint new_game_2_cb (GtkWidget *widget, gpointer data)
+static gint
+new_game_2_cb (GtkWidget *widget, gpointer data)
 {
 	if (!paused) {
 		if (!keyboard_id)
 			keyboard_id = g_signal_connect (G_OBJECT (window),
-					"key_press_event",
-					G_CALLBACK (key_press_cb), NULL);
+							"key_press_event",
+							G_CALLBACK (key_press_cb),
+							NULL);
 		if (!main_id)
 			main_id = gtk_timeout_add (GAMEDELAY * properties->gamespeed,
-					(GtkFunction) main_loop, NULL);
+						   (GtkFunction) main_loop,
+						   NULL);
 		if (!add_bonus_id)
 			add_bonus_id = gtk_timeout_add (BONUSDELAY *
-					properties->gamespeed,
-					(GtkFunction) add_bonus_cb,
-					NULL);
+							properties->gamespeed,
+							(GtkFunction) add_bonus_cb,
+							NULL);
 	}
 
 	dummy_id = 0;
@@ -322,7 +297,8 @@ static gint new_game_2_cb (GtkWidget *widget, gpointer data)
 	return (FALSE);
 }
 
-static gint new_game_cb (GtkWidget *widget, gpointer data)
+static gint
+new_game_cb (GtkWidget *widget, gpointer data)
 {
 	gtk_widget_set_sensitive (game_menu[1].widget, TRUE);
 	gtk_widget_set_sensitive (game_menu[4].widget, TRUE);
@@ -331,11 +307,8 @@ static gint new_game_cb (GtkWidget *widget, gpointer data)
 	*/
 
 	if (game_running ()) {
-		if (!end_game_box (0)) { // 0 for new game
 			end_game_cb (widget, (gpointer) 0);
 			main_id = 0;
-		} else
-			return 0;
 	}
 
 	gnibbles_init ();
@@ -370,6 +343,8 @@ static gint new_game_cb (GtkWidget *widget, gpointer data)
 		gtk_timeout_remove (dummy_id);
 
 	dummy_id = gtk_timeout_add (1500, (GtkFunction) new_game_2_cb, NULL);
+
+	return TRUE;
 }
 
 gint pause_game_cb (GtkWidget *widget, gpointer data)
@@ -407,6 +382,7 @@ gint pause_game_cb (GtkWidget *widget, gpointer data)
 			}
 		}
 	}
+	return TRUE;
 }
 
 static gint show_scores_cb (GtkWidget *widget, gpointer data)
@@ -414,11 +390,9 @@ static gint show_scores_cb (GtkWidget *widget, gpointer data)
 	gnibbles_show_scores (window, 0);
 }
 
-static gint end_game_cb (GtkWidget *widget, gpointer data)
+void
+end_game (gint data)
 {
-	if ((gint) data == 2 && end_game_box (0)) // 0 for new game
-			return 0;
-
 	if (main_id) {
 		gtk_timeout_remove (main_id);
 		main_id = 0;
@@ -459,7 +433,11 @@ static gint end_game_cb (GtkWidget *widget, gpointer data)
 	}
 
 	paused = 0;
+}
 
+static gint end_game_cb (GtkWidget *widget, gpointer data)
+{
+	end_game ((gint)data);
 	return (FALSE);
 }
 
@@ -516,8 +494,8 @@ static gint main_loop (gpointer data)
 		gtk_timeout_remove (add_bonus_id);
 		add_bonus_id = 0;
 		erase_id = gtk_timeout_add (3000,
-				(GtkFunction) erase_worms_cb,
-				(gpointer) ERASESIZE);
+					    (GtkFunction) erase_worms_cb,
+					    (gpointer) ERASESIZE);
 		gnibbles_log_score (window);
 		return (FALSE);
 	}
@@ -558,48 +536,54 @@ static gint main_loop (gpointer data)
 	return (TRUE);
 }
 
-static void set_bg_color ()
+static void
+set_bg_color (void)
 {
 	GdkImage *tmp_image;
 	GdkColor bgcolor;
 
-	tmp_image = gdk_drawable_get_image (GDK_DRAWABLE (gnibbles_pixmap), 0, 0, 1, 1);
+	tmp_image = gdk_drawable_get_image (GDK_DRAWABLE (gnibbles_pixmap),
+					    0, 0, 1, 1);
 	bgcolor.pixel = gdk_image_get_pixel (tmp_image, 0, 0);
 	gdk_window_set_background (drawing_area->window, &bgcolor);
 	g_object_unref (G_OBJECT (tmp_image));
 }
 
-void update_score_state ()
+void
+update_score_state (void)
 {
         gchar **names = NULL;
         gfloat *scores = NULL;
         time_t *scoretimes = NULL;
 	gint top;
 
-	char buf[10];
-	sprintf (buf, "%d.%d", properties->gamespeed, properties->fakes);
+	gchar *buf = NULL;
+	buf = g_strdup_printf ("%d.%d", properties->gamespeed,
+			       properties->fakes);
 
-	top = gnome_score_get_notable("gnibbles", buf, &names, &scores, &scoretimes);
+	top = gnome_score_get_notable ("gnibbles", buf,
+				       &names, &scores, &scoretimes);
+	g_free (buf);
 	if (top > 0) {
 		gtk_widget_set_sensitive (game_menu[3].widget, TRUE);
-		g_strfreev(names);
-		g_free(scores);
-		g_free(scoretimes);
+		g_strfreev (names);
+		g_free (scores);
+		g_free (scoretimes);
 	} else {
 		gtk_widget_set_sensitive (game_menu[3].widget, FALSE);
 	}
 }
 
-gboolean
+static gboolean
 show_cursor_cb (GtkWidget *widget, GdkEventMotion *event, gpointer data)
 {
         show_cursor ();
         return FALSE;
 }
 
-static void setup_window ()
+static void
+setup_window (void)
 {
-	GtkWidget *label, *hbox;
 	GdkPixmap *cursor_dot_pm;
 
 	window = gnome_app_new ("gnibbles", "GNOME Nibbles");
@@ -632,7 +616,8 @@ static void setup_window ()
 			G_CALLBACK (show_cursor_cb), NULL);
 
 	gtk_widget_set_size_request (GTK_WIDGET (drawing_area),
-			BOARDWIDTH*properties->tilesize, BOARDHEIGHT*properties->tilesize);
+				     BOARDWIDTH*properties->tilesize,
+				     BOARDHEIGHT*properties->tilesize);
 	g_signal_connect (G_OBJECT (drawing_area), "expose_event",
 			G_CALLBACK (expose_event_cb), NULL);
 	gtk_widget_set_events (drawing_area, GDK_BUTTON_PRESS_MASK |
@@ -648,37 +633,68 @@ static void setup_window ()
 	scoreboard = gnibbles_scoreboard_new (appbar);
 }
 
-static void load_properties ()
+static void
+gconf_key_change_cb (GConfClient *client, guint cnxn_id,
+		     GConfEntry *entry, gpointer user_data)
 {
+	end_game (1);
+	gnibbles_properties_destroy (properties);
 	properties = gnibbles_properties_new ();
 }
 
-static void render_logo ()
+static void
+load_properties ()
 {
-	gint i, j;
+	gint i;
+	gchar *buffer;
 
+	properties = gnibbles_properties_new ();
+
+	/* maybe this should to into properties.c */
+        gconf_client_add_dir (gconf_client_get_default (),
+                              KEY_DIR,
+                              GCONF_CLIENT_PRELOAD_RECURSIVE,
+                              NULL);
+	gconf_client_notify_add (gconf_client_get_default (),
+				 KEY_PREFERENCES_DIR,
+				 gconf_key_change_cb, NULL, NULL, NULL);
+	for (i = 0; i < NUMWORMS; i++) {
+		buffer = g_strdup_printf (KEY_WORM_DIR, i);
+		gconf_client_notify_add (gconf_client_get_default (),
+					 buffer,
+					 gconf_key_change_cb, NULL, NULL, NULL);
+		g_free (buffer);
+	}
+}
+
+static void
+render_logo (void)
+{
 	zero_board ();
 
 	gdk_draw_drawable (GDK_DRAWABLE (buffer_pixmap),
-			drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)], logo_pixmap,
-      0, 0, 0, 0, BOARDWIDTH*properties->tilesize,
-      BOARDHEIGHT*properties->tilesize);
+			   drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)],
+			   logo_pixmap,
+			   0, 0, 0, 0,
+			   BOARDWIDTH * properties->tilesize,
+			   BOARDHEIGHT * properties->tilesize);
 
 	gdk_draw_drawable (GDK_DRAWABLE (drawing_area->window),
-			drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)],
-      buffer_pixmap, 0, 0, 0, 0, BOARDWIDTH*properties->tilesize,
-      BOARDHEIGHT*properties->tilesize);
+			   drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)],
+			   buffer_pixmap,
+			   0, 0, 0, 0,
+			   BOARDWIDTH * properties->tilesize,
+			   BOARDHEIGHT * properties->tilesize);
 }
 
-int main (int argc, char **argv)
+int 
+main (int argc, char **argv)
 {
-	gint foo;
-
 	gnome_score_init ("gnibbles");
 
-	bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-	textdomain(GETTEXT_PACKAGE);
+	textdomain (GETTEXT_PACKAGE);
 	
 	gnome_program_init ("gnibbles", VERSION, LIBGNOMEUI_MODULE,
 			    argc, argv,
@@ -699,8 +715,9 @@ int main (int argc, char **argv)
 	gtk_widget_show (window);
 
 	buffer_pixmap = gdk_pixmap_new (drawing_area->window,
-			BOARDWIDTH * properties->tilesize, BOARDHEIGHT * properties->tilesize,
-			-1);
+					BOARDWIDTH * properties->tilesize,
+					BOARDHEIGHT * properties->tilesize,
+					-1);
 
 	render_logo ();
 
