@@ -1,3 +1,5 @@
+/* -*- mode:C; tab-width:8; c-basic-offset:8; indent-tabs-mode:true -*- */
+
 /*
  *
  * Author:        Pista <szekeres@cyberspace.mht.bme.hu>
@@ -14,6 +16,7 @@
 
 #include "minefield.h"
 #include "games-clock.h"
+#include "games-frame.h"
 
 /* Limits for various minefield properties */
 #define MINESIZE_MIN 2
@@ -30,9 +33,10 @@
 #define KEY_NMINES "/apps/gnomine/geometry/nmines"
 #define KEY_MINESIZE "/apps/gnomine/geometry/minesize"
 #define KEY_MODE "/apps/gnomine/geometry/mode"
+#define KEY_USE_QUESTION_MARKS "/apps/gnomine/use_question_marks"
 
 static GtkWidget *mfield;
-static GtkWidget *pref_dialog;
+static GtkWidget *pref_dialog = NULL;
 static GtkWidget *rbutton;
 static GtkWidget *ralign;
 static GConfClient *conf_client;
@@ -42,6 +46,7 @@ GtkWidget *xentry;
 GtkWidget *yentry;
 GtkWidget *mentry;
 GtkWidget *sentry;
+GtkWidget *question_toggle;
 GtkWidget *mbutton;
 GtkWidget *plabel;
 GtkWidget *cframe;
@@ -52,6 +57,7 @@ gint ysize = -1, xsize = -1;
 gint nmines = -1;
 gint fsize = -1;
 gint minesize = -1;
+gboolean use_question_marks = TRUE;
 
 GnomeUIInfo gamemenu[];
 GnomeUIInfo settingsmenu[];
@@ -65,6 +71,9 @@ char *fsize2names[] = {
 	N_("Custom"),
 };
 
+static void create_preferences (void);
+
+
 static GtkWidget *
 image_widget_setup (char *name)
 {
@@ -72,8 +81,8 @@ image_widget_setup (char *name)
 	char *filename = NULL;
 
 	filename = gnome_program_locate_file (NULL,
-			GNOME_FILE_DOMAIN_APP_PIXMAP, name,
-			TRUE, NULL);
+					      GNOME_FILE_DOMAIN_APP_PIXMAP, name,
+					      TRUE, NULL);
 	if (filename != NULL)
 		pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
 
@@ -82,88 +91,113 @@ image_widget_setup (char *name)
 	return gtk_image_new_from_pixbuf (pixbuf);
 }
 
-void show_face(GtkWidget *pm)
+static void
+show_face (GtkWidget *pm)
 {
         if (pm_current == pm) return;
 
 	if (pm_current) {
-		gtk_widget_hide(pm_current);
+		gtk_widget_hide (pm_current); 
 	}
 
-	gtk_widget_show(pm);
+	gtk_widget_show (pm);
 	
 	pm_current = pm;
 }
 
-void quit_game(GtkWidget *widget, gpointer data)
+static void
+quit_game (GtkWidget *widget, gpointer data)
 {
-	gtk_main_quit();
+	gtk_main_quit ();
 }
 
-void set_flabel(GtkMineField *mfield)
+static void
+set_flabel (GtkMineField *mfield)
 {
 	char *val;
 
 	val = g_strdup_printf ("%d/%d", mfield->flag_count, mfield->mcount);
-	gtk_label_set_text (GTK_LABEL(flabel), val);
+	gtk_label_set_text (GTK_LABEL (flabel), val);
 }
 
-void update_score_state ()
+static void
+update_score_state (void)
 {
         gchar **names = NULL;
         gfloat *scores = NULL;
-       time_t *scoretimes = NULL;
+	time_t *scoretimes = NULL;
 	gint top;
 	gchar buf[64];
 
-	if(fsize<4)
-		strncpy(buf, fsize2names[fsize], sizeof(buf));
+	if (fsize<4)
+		strncpy (buf, fsize2names[fsize], sizeof(buf));
 
-	top = gnome_score_get_notable("gnomine", buf, &names, &scores, &scoretimes);
+	top = gnome_score_get_notable ("gnomine", buf, &names, &scores, &scoretimes);
 	if (top > 0) {
 		gtk_widget_set_sensitive (gamemenu[2].widget, TRUE);
-		g_strfreev(names);
-		g_free(scores);
-		g_free(scoretimes);
-	} else {
+		g_strfreev (names);
+		g_free (scores);
+		g_free (scoretimes);
+	} else { 
 		gtk_widget_set_sensitive (gamemenu[2].widget, FALSE);
 	}
 }
 
-void
+static void
 show_scores (gchar *level, guint pos)
 {
 	gnome_scores_display (_("GNOME Mines"), "gnomine", level, pos);
 }
 
-void top_ten(GtkWidget *widget, gpointer data)
+static void
+top_ten (GtkWidget *widget, gpointer data)
 {
 	gchar buf[64];
 
 	if(fsize<4)
-		strncpy(buf, fsize2names[fsize], sizeof(buf));
+		strncpy (buf, fsize2names[fsize], sizeof(buf));
 
-	show_scores(buf, 0);
+	show_scores (buf, 0);
 }
 
-void new_game(GtkWidget *widget, gpointer data)
+static void
+new_game (GtkWidget *widget, gpointer data)
 {
-	games_clock_stop(GAMES_CLOCK(clk));
-	games_clock_set_seconds(GAMES_CLOCK(clk), 0);
+	games_clock_stop (GAMES_CLOCK (clk));
+	games_clock_set_seconds (GAMES_CLOCK (clk), 0);
 
-	show_face(pm_smile);
-	gtk_minefield_restart(GTK_MINEFIELD(mfield));
-	set_flabel(GTK_MINEFIELD(mfield));
+	show_face (pm_smile);
+	gtk_minefield_restart (GTK_MINEFIELD (mfield));
+	set_flabel (GTK_MINEFIELD (mfield));
 
 	gtk_widget_hide (ralign);
-	gtk_widget_show (mfield);
+	gtk_widget_show (mfield); 
 }
 
-void focus_out_cb (GtkWidget *widget, GdkEventFocus *event, gpointer data)
+static gboolean
+configure_cb (GtkWidget *widget, GdkEventConfigure *event)
+{
+	GtkMineField *mf = GTK_MINEFIELD (mfield);
+	gint size = MIN (event->width / mf->xsize, event->height / mf->ysize);
+
+	/* problem? configure_cb expands minefield to fit the window, so it
+	   does not allow you to reduce (in preferences) minefield smaller than
+	   the statusbar min width */
+	if (mf->minesize != size) {
+		if (pref_dialog == NULL)
+			create_preferences ();
+		/* make sure preferences dialog updates the minesize value */
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (sentry), size);
+	}
+	return FALSE;
+}
+
+static void
+focus_out_cb (GtkWidget *widget, GdkEventFocus *event, gpointer data)
 {
         GtkRequisition req;
 
-	if (GAMES_CLOCK(clk)->timer_id == -1)
+	if (GAMES_CLOCK (clk)->timer_id == -1)
 		return;
 
         /* This is a complete abuse of the sizing system, but it seems to
@@ -175,83 +209,90 @@ void focus_out_cb (GtkWidget *widget, GdkEventFocus *event, gpointer data)
 	gtk_widget_hide (mfield);
 	gtk_widget_show (ralign);
 
-	gtk_widget_grab_focus(rbutton);
+	gtk_widget_grab_focus (rbutton);
 
-	games_clock_stop(GAMES_CLOCK(clk)); 
+	games_clock_stop (GAMES_CLOCK (clk)); 
 }
 
-void resume_game_cb (GtkButton *widget, gpointer data)
+static void
+resume_game_cb (GtkButton *widget, gpointer data)
 {
 	gtk_widget_hide (ralign);
 	gtk_widget_show (mfield);
 
-	games_clock_start(GAMES_CLOCK(clk));
+	games_clock_start (GAMES_CLOCK (clk));
 }
 
-void marks_changed(GtkWidget *widget, gpointer data)
+static void
+marks_changed (GtkWidget *widget, gpointer data)
 {
-        set_flabel(GTK_MINEFIELD(widget));
+        set_flabel (GTK_MINEFIELD (widget));
 
-	games_clock_start(GAMES_CLOCK(clk));
+	games_clock_start (GAMES_CLOCK (clk));
 }
 
-void lose_game(GtkWidget *widget, gpointer data)
+static void
+lose_game (GtkWidget *widget, gpointer data)
 {
-        show_face(pm_sad);
+        show_face (pm_sad);
 
-	gtk_widget_grab_focus(mbutton);
+	gtk_widget_grab_focus (mbutton);
 
-        games_clock_stop(GAMES_CLOCK(clk));
+        games_clock_stop (GAMES_CLOCK (clk));
 }
 
-void win_game(GtkWidget *widget, gpointer data)
+static void
+win_game (GtkWidget *widget, gpointer data)
 {
         gfloat score;
         int pos;
         gchar buf[64];
 
 
-        games_clock_stop(GAMES_CLOCK(clk));
+        games_clock_stop (GAMES_CLOCK (clk));
 
-	gtk_widget_grab_focus(mbutton);
+	gtk_widget_grab_focus (mbutton);
 
-        show_face(pm_win);
+        show_face (pm_win);
 
 	if(fsize<4) {
-	    score = (gfloat) (GAMES_CLOCK(clk)->stopped / 60) + 
-		    (gfloat) (GAMES_CLOCK(clk)->stopped % 60) / 100;
+	    score = (gfloat) (GAMES_CLOCK (clk)->stopped / 60) + 
+		    (gfloat) (GAMES_CLOCK (clk)->stopped % 60) / 100;
 
             strncpy(buf, fsize2names[fsize], sizeof(buf));
-	    pos = gnome_score_log(score, buf, FALSE);
+	    pos = gnome_score_log (score, buf, FALSE);
 	} else {
 	    score = ((nmines * 100) / (xsize * ysize)) /
-		    (gfloat) (GAMES_CLOCK(clk)->stopped); 
+		    (gfloat) (GAMES_CLOCK (clk)->stopped); 
 
             strncpy(buf, fsize2names[fsize], sizeof(buf));
-	    pos = gnome_score_log(score, buf, TRUE);
+	    pos = gnome_score_log (score, buf, TRUE);
 	}
 
 	update_score_state ();
 
-	show_scores(buf, pos);
+	show_scores (buf, pos);
 }
 
-void look_cell(GtkWidget *widget, gpointer data)
+static void
+look_cell (GtkWidget *widget, gpointer data)
 {
-        show_face(pm_worried);
+        show_face (pm_worried);
 
-        games_clock_start(GAMES_CLOCK(clk));
+        games_clock_start (GAMES_CLOCK (clk));
 }
 
-void unlook_cell(GtkWidget *widget, gpointer data)
+static void
+unlook_cell (GtkWidget *widget, gpointer data)
 {
 	show_face(pm_cool);
 }
 
-void setup_mode(GtkWidget *widget, gint mode)
+static void
+setup_mode (GtkWidget *widget, gint mode)
 {
-	gint size_table[3][3] = {{ 8, 8, 10 }, {16, 16, 40}, {30, 16, 99}};
-	gint x,y,m,s;
+	static gint size_table[3][3] = {{ 8, 8, 10 }, {16, 16, 40}, {30, 16, 99}};
+	gint x,y,m;
 
 	if (mode == 3) {
 		x = xsize;
@@ -262,12 +303,12 @@ void setup_mode(GtkWidget *widget, gint mode)
 		y = size_table[mode][1];
 		m = size_table[mode][2];
 	}
-	s = minesize;
-	gtk_minefield_set_size(GTK_MINEFIELD(mfield), x, y);
-	gtk_minefield_set_mines(GTK_MINEFIELD(mfield), m, s);
+	gtk_minefield_set_size (GTK_MINEFIELD (mfield), x, y);
+	gtk_minefield_set_mines (GTK_MINEFIELD (mfield), m, minesize); 
 }
 
-int range (int val, int min, int max)
+static int
+range (int val, int min, int max)
 {
 	if (val < min)
 		val = min;
@@ -277,7 +318,8 @@ int range (int val, int min, int max)
 }
 
 
-void verify_ranges (void)
+static void
+verify_ranges (void)
 {
 	minesize = range (minesize, MINESIZE_MIN, MINESIZE_MAX);
 	xsize    = range (xsize, XSIZE_MIN, XSIZE_MAX);
@@ -285,16 +327,21 @@ void verify_ranges (void)
 	nmines   = range (nmines, 1, xsize * ysize);
 }
 
-void
-about(GtkWidget *widget, gpointer data)
+static void
+about (GtkWidget *widget, gpointer data)
 {
         static GtkWidget *about;
 	GdkPixbuf *pixbuf = NULL;
 
         const gchar *authors[] = {
-		N_("Code: Pista"),
-		N_("Faces: tigert"),
-		N_("Score: HoraPe"),
+		_("Main game:"),
+		"Szekeres Istvan",
+		"",
+		_("Faces:"),
+		"Tuomas Kuosmanen",
+		"",
+		_("Score:"),
+		"Horacio J. Pe\xc3\xb1""a",
 		NULL
 	};
 	const gchar *documenters[] = {
@@ -325,15 +372,16 @@ about(GtkWidget *widget, gpointer data)
        }
 
         about = gnome_about_new (_("GNOME Mines"), VERSION,
-				 _("(C) 1997-1999 the Free Software Foundation"),
-				 _("Minesweeper clone"),
+				 "Copyright \xc2\xa9 1997-2003 Free Software "
+				 "Foundation, Inc.",
+				 _("A Minesweeper clone."),
 				 (const char **)authors,
 				 (const char **)documenters,
 				 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
 				 pixbuf);
 	
 	if (pixbuf != NULL)
-		gdk_pixbuf_unref (pixbuf);
+		g_object_unref (pixbuf);
 	
 	g_signal_connect (GTK_OBJECT (about), "destroy", GTK_SIGNAL_FUNC
 			(gtk_widget_destroyed), &about);
@@ -383,7 +431,9 @@ gconf_key_change_cb (GConfClient *client, guint cnxn_id,
 		i = value ? gconf_value_get_int (value) : 17;
 		if (minesize != i) {
 			minesize = range (i, MINESIZE_MIN, MINESIZE_MAX);
-			gtk_minefield_set_mines(GTK_MINEFIELD(mfield), nmines, minesize);
+			gtk_minefield_set_mines(GTK_MINEFIELD(mfield),
+						GTK_MINEFIELD(mfield)->mcount,
+						minesize);
 		}
 	}
 	if (strcmp (key, KEY_MODE) == 0) {
@@ -396,10 +446,16 @@ gconf_key_change_cb (GConfClient *client, guint cnxn_id,
 			new_game (mfield, NULL);
 		}
 	}
+	if (strcmp (key, KEY_USE_QUESTION_MARKS) == 0) {
+		use_question_marks = value ? gconf_value_get_bool (value) : TRUE;
+		gtk_minefield_set_use_question_marks(GTK_MINEFIELD(mfield), use_question_marks);
+	}
+	/* now window is resizable we need to shrink it when changing minesize etc */
+	gtk_window_resize (GTK_WINDOW (window), 1, 1);
 }
 
 static void
-size_radio_callback(GtkWidget *widget, gpointer data)
+size_radio_callback (GtkWidget *widget, gpointer data)
 {
 	int fsc;
 	if (!pref_dialog)
@@ -413,7 +469,8 @@ size_radio_callback(GtkWidget *widget, gpointer data)
 	gtk_widget_set_sensitive(cframe, fsc == 3);
 }
 
-void fix_nmines (int xsize, int ysize)
+static void
+fix_nmines (int xsize, int ysize)
 {
   int maxmines;
 
@@ -464,7 +521,16 @@ minesize_spin_cb (GtkSpinButton *spin, gpointer data)
 
 }
 
-static void preferences_callback (GtkWidget *widget, gpointer data)
+static void
+use_question_toggle_cb (GtkCheckButton *check, gpointer data)
+{
+	gboolean use_marks = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check));
+	gconf_client_set_bool (conf_client, KEY_USE_QUESTION_MARKS,
+				use_marks, NULL);
+}
+
+static void
+create_preferences (void)
 {
 	GtkWidget *table;
 	GtkWidget *frame;
@@ -473,31 +539,16 @@ static void preferences_callback (GtkWidget *widget, gpointer data)
 	GtkWidget *button;
 	GtkWidget *table2;
 	GtkWidget *label2;
-	char *numstr;
-        gchar * label;
         
-	if (pref_dialog) {
-                gtk_window_present (GTK_WINDOW (pref_dialog));
-		return;
-        }
-
-	table = gtk_table_new (2, 2, FALSE);
+	table = gtk_table_new (2, 3, FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (table), GNOME_PAD);
-	gtk_widget_show (table);
 	gtk_table_set_row_spacings (GTK_TABLE (table), GNOME_PAD);
 	gtk_table_set_col_spacings (GTK_TABLE (table), GNOME_PAD);
 
-        label = g_strdup_printf("<b>%s</b>",_("Field size"));
-	frame = gtk_frame_new (label);
-        g_free(label);
-	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
-	gtk_label_set_use_markup (GTK_LABEL (gtk_frame_get_label_widget (GTK_FRAME (frame))), TRUE);
-	gtk_container_set_border_width (GTK_CONTAINER (frame), 0);
-	gtk_widget_show (frame);
+	frame = games_frame_new (_("Field size"));
 
 	vbox = gtk_vbox_new (TRUE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD);
-	gtk_widget_show (vbox);
 
 	button = gtk_radio_button_new_with_label(NULL, _("Small"));
 	if (fsize == 0)
@@ -506,7 +557,6 @@ static void preferences_callback (GtkWidget *widget, gpointer data)
 	g_signal_connect (GTK_OBJECT (button), "clicked",
 			GTK_SIGNAL_FUNC (size_radio_callback), (gpointer) 0);
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
-        gtk_widget_show(button);
 
 	button = gtk_radio_button_new_with_label
 		(gtk_radio_button_get_group (GTK_RADIO_BUTTON(button)),
@@ -517,7 +567,6 @@ static void preferences_callback (GtkWidget *widget, gpointer data)
 	g_signal_connect (GTK_OBJECT (button), "clicked",
 			GTK_SIGNAL_FUNC (size_radio_callback), (gpointer) 1);
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
-        gtk_widget_show (button);
 
 	button = gtk_radio_button_new_with_label
 		(gtk_radio_button_get_group (GTK_RADIO_BUTTON (button)),
@@ -528,7 +577,6 @@ static void preferences_callback (GtkWidget *widget, gpointer data)
 	g_signal_connect (GTK_OBJECT (button), "clicked",
 			GTK_SIGNAL_FUNC (size_radio_callback), (gpointer) 2);
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
-        gtk_widget_show(button);
 	
 	button = gtk_radio_button_new_with_label
 		(gtk_radio_button_get_group(GTK_RADIO_BUTTON(button)),
@@ -539,30 +587,21 @@ static void preferences_callback (GtkWidget *widget, gpointer data)
 	g_signal_connect (GTK_OBJECT (button), "clicked",
 			GTK_SIGNAL_FUNC (size_radio_callback), (gpointer) 3);
 	gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, TRUE, 0);
-        gtk_widget_show(button);
 
 	gtk_container_add (GTK_CONTAINER (frame), vbox);
 
 	gtk_table_attach (GTK_TABLE (table), frame, 0, 1, 0, 1, GTK_EXPAND |
 			GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 
-        label = g_strdup_printf("<b>%s</b>",_("Custom size"));
-	cframe = gtk_frame_new (label);
-        g_free(label);
-	gtk_frame_set_shadow_type (GTK_FRAME (cframe), GTK_SHADOW_NONE);
-	gtk_label_set_use_markup (GTK_LABEL (gtk_frame_get_label_widget (GTK_FRAME (cframe))), TRUE);
-	gtk_container_set_border_width (GTK_CONTAINER (cframe), 0);
+	cframe = games_frame_new (_("Custom size"));
 	gtk_widget_set_sensitive (cframe, fsize == 3);
-	gtk_widget_show (cframe);
 
 	table2 = gtk_table_new (3, 2, FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (table2), GNOME_PAD);
-	gtk_widget_show (table2);
 	gtk_table_set_row_spacings (GTK_TABLE (table2), GNOME_PAD);
 
 	label2 = gtk_label_new (_("Number of mines:"));
 	gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);
-	gtk_widget_show (label2);
 	gtk_table_attach (GTK_TABLE (table2), label2, 0, 1, 2, 3,
 			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 	/* TODO: fix this when gconfised correctly */
@@ -572,11 +611,9 @@ static void preferences_callback (GtkWidget *widget, gpointer data)
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON(mentry), nmines);
 	gtk_table_attach (GTK_TABLE (table2), mentry, 1, 2, 2, 3, GTK_FILL, GTK_EXPAND
 			| GTK_FILL, 0, 0);
-	gtk_widget_show (mentry);
 
 	label2 = gtk_label_new (_("Horizontal:"));
 	gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);
-	gtk_widget_show (label2);
 	gtk_table_attach (GTK_TABLE (table2), label2, 0, 1, 0, 1,
 			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 
@@ -586,11 +623,9 @@ static void preferences_callback (GtkWidget *widget, gpointer data)
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON(xentry), xsize);
 	gtk_table_attach (GTK_TABLE (table2), xentry, 1, 2, 0, 1, 0, GTK_EXPAND
 			| GTK_FILL, 0, 0);
-	gtk_widget_show (xentry);
 
 	label2 = gtk_label_new (_("Vertical:"));
 	gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);
-	gtk_widget_show (label2);
 	gtk_table_attach (GTK_TABLE (table2), label2, 0, 1, 1, 2,
 			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 
@@ -600,7 +635,6 @@ static void preferences_callback (GtkWidget *widget, gpointer data)
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON(yentry), ysize);
 	gtk_table_attach (GTK_TABLE (table2), yentry, 1, 2, 1, 2, 0, GTK_EXPAND
 			| GTK_FILL, 0, 0);
-	gtk_widget_show (yentry);
 
 	gtk_container_add (GTK_CONTAINER (cframe), table2);
 
@@ -608,37 +642,53 @@ static void preferences_callback (GtkWidget *widget, gpointer data)
 			GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 
 	hbox = gtk_hbox_new (FALSE, GNOME_PAD);
-	gtk_widget_show (hbox);
 
 	label2 = gtk_label_new(_("Mine size:"));
 	gtk_box_pack_start (GTK_BOX (hbox), label2, FALSE, FALSE, 0);
-	gtk_widget_show (label2);
 
 	sentry = gtk_spin_button_new_with_range(MINESIZE_MIN, MINESIZE_MAX, 1);
 	g_signal_connect (GTK_OBJECT (sentry), "value-changed",
 			GTK_SIGNAL_FUNC (minesize_spin_cb), NULL);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON(sentry), minesize);
 	gtk_box_pack_start (GTK_BOX (hbox), sentry, TRUE, TRUE, 0);
-	gtk_widget_show(sentry);
 
 	gtk_table_attach (GTK_TABLE (table), hbox, 0, 1, 1, 2, GTK_EXPAND |
 			GTK_FILL, 0, 0, 0);
 
+	question_toggle = gtk_check_button_new_with_label(_("Use \"I'm not sure\" flags."));
+	g_signal_connect(GTK_OBJECT(question_toggle), "toggled",
+			GTK_SIGNAL_FUNC (use_question_toggle_cb), NULL);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(question_toggle), use_question_marks);
+	gtk_widget_show(question_toggle);
+	
+	gtk_table_attach (GTK_TABLE (table), question_toggle, 0, 2, 2, 3, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+
+
 	pref_dialog = gtk_dialog_new_with_buttons (_("GNOME Mines Preferences"),
 			GTK_WINDOW (window),
 			0,
 			GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 			NULL);
 
+        gtk_dialog_set_has_separator (GTK_DIALOG (pref_dialog), FALSE);
 	gtk_container_add (GTK_CONTAINER(GTK_DIALOG(pref_dialog)->vbox),
 			table);
 
-	g_signal_connect (GTK_OBJECT (pref_dialog), "destroy",
-			GTK_SIGNAL_FUNC (gtk_widget_destroyed), &pref_dialog);
-	g_signal_connect (GTK_OBJECT (pref_dialog), "response",
-			GTK_SIGNAL_FUNC (gtk_widget_destroy), &pref_dialog);
+	g_signal_connect (G_OBJECT (pref_dialog), "destroy",
+			  G_CALLBACK (gtk_widget_destroyed), &pref_dialog);
+	g_signal_connect (G_OBJECT (pref_dialog), "response",
+			  G_CALLBACK (gtk_widget_hide), NULL);
 
-	gtk_widget_show (pref_dialog);
+	/* show all child widgets, but do not display the dialog (yet) */
+	gtk_widget_show_all (GTK_WIDGET (table));
+}
+
+static void
+preferences_callback (GtkWidget *widget, gpointer data)
+{
+	if (pref_dialog == NULL)
+		create_preferences ();
+	gtk_window_present (GTK_WINDOW (pref_dialog));
 }
 
 GnomeUIInfo gamemenu[] = {
@@ -737,7 +787,7 @@ main (int argc, char *argv[])
 {
         GnomeAppBar *appbar;
         GtkWidget *all_boxes;
-	GtkWidget *status_table;
+	GtkWidget *status_box;
 	GtkWidget *button_table;
         GtkWidget *align;
         GtkWidget *box;        
@@ -763,79 +813,85 @@ main (int argc, char *argv[])
                               GCONF_CLIENT_PRELOAD_RECURSIVE,
                               NULL);
 	gconf_client_notify_add (conf_client, KEY_DIR,
-				gconf_key_change_cb, NULL, NULL, NULL);
+				 gconf_key_change_cb, NULL, NULL, NULL);
 
 	gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-gnomine.png");
 	client = gnome_master_client ();
-	g_signal_connect (GTK_OBJECT (client), "save_yourself",
-			    GTK_SIGNAL_FUNC (save_state), argv[0]);
+	g_signal_connect (G_OBJECT (client), "save_yourself",
+			  G_CALLBACK (save_state), argv[0]);
 
 	if (xpos > 0 || ypos > 0)
-	  gdk_window_move (GTK_WIDGET(window)->window, xpos, ypos);
+	  gdk_window_move (GTK_WIDGET (window)->window, xpos, ypos);
 		
 	if (xsize == -1)
 		xsize = gconf_client_get_int (conf_client,
-				KEY_XSIZE,
-				NULL);
+					      KEY_XSIZE,
+					      NULL);
 	if (ysize == -1)
 		ysize = gconf_client_get_int (conf_client,
-				KEY_YSIZE,
-				NULL);
+					      KEY_YSIZE,
+					      NULL);
 	if (nmines == -1)
 		nmines = gconf_client_get_int (conf_client,
-				KEY_NMINES,
-				NULL);
+					       KEY_NMINES,
+					       NULL);
 	if (minesize == -1)
 		minesize = gconf_client_get_int (conf_client,
-				KEY_MINESIZE,
-				NULL);
+						 KEY_MINESIZE,
+						 NULL);
 	if (fsize == -1)
 		fsize = gconf_client_get_int (conf_client,
-				KEY_MODE,
-				NULL);
+					      KEY_MODE,
+					      NULL);
+	use_question_marks = gconf_client_get_bool (conf_client, 
+						    KEY_USE_QUESTION_MARKS,
+						    NULL);
+	
 	verify_ranges ();
 
 #define ELEMENTS(x) (sizeof(x) / sizeof(x[0])) 
 	{
 		int i;
 		for (i = 0; i < ELEMENTS(mainmenu); i++)
-			mainmenu[i].label = gettext(mainmenu[i].label);
+			mainmenu[i].label = gettext (mainmenu[i].label);
 		for (i = 0; i < ELEMENTS(gamemenu); i++)
-			gamemenu[i].label = gettext(gamemenu[i].label);
+			gamemenu[i].label = gettext (gamemenu[i].label);
 		for (i = 0; i < ELEMENTS(helpmenu); i++)
-			helpmenu[i].label = gettext(helpmenu[i].label);
+			helpmenu[i].label = gettext (helpmenu[i].label);
 	}
 
-	window = gnome_app_new("gnomine", _("GNOME Mines"));
-	gnome_app_create_menus(GNOME_APP(window), mainmenu);
-	gtk_window_set_resizable (GTK_WINDOW(window), FALSE);
+	window = gnome_app_new ("gnomine", _("GNOME Mines"));
+	gnome_app_create_menus (GNOME_APP (window), mainmenu);
+	gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
 
-	appbar = GNOME_APPBAR (gnome_appbar_new(FALSE, FALSE, FALSE));
-	gnome_app_set_statusbar(GNOME_APP (window), GTK_WIDGET (appbar));
+	appbar = GNOME_APPBAR (gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_NEVER));
+	gnome_app_set_statusbar (GNOME_APP (window), GTK_WIDGET (appbar));
 	
-	g_signal_connect(GTK_OBJECT(window), "delete_event",
-			    GTK_SIGNAL_FUNC(quit_game), NULL);
-	g_signal_connect(GTK_OBJECT(window), "focus_out_event",
-			    GTK_SIGNAL_FUNC(focus_out_cb), NULL);
+	g_signal_connect(G_OBJECT (window), "delete_event",
+			 G_CALLBACK (quit_game), NULL);
+	g_signal_connect(G_OBJECT (window), "focus_out_event",
+			 G_CALLBACK (focus_out_cb), NULL);
+	g_signal_connect (GTK_OBJECT (window), "configure_event",
+			  G_CALLBACK (configure_cb), NULL);
 
-	all_boxes = gtk_vbox_new(FALSE, 0);
+	all_boxes = gtk_vbox_new (FALSE, 0);
 
-	gnome_app_set_contents(GNOME_APP(window), all_boxes);
+	gnome_app_set_contents (GNOME_APP (window), all_boxes);
 
-        	button_table = gtk_table_new(2, 3, FALSE);
-	gtk_box_pack_start(GTK_BOX(all_boxes), button_table, TRUE, TRUE, 0);
+        	button_table = gtk_table_new (2, 3, FALSE);
+	gtk_box_pack_start (GTK_BOX (all_boxes), button_table, TRUE, TRUE, 0);
 
         	pm_current = NULL;
 
-	mbutton = gtk_button_new();
-	g_signal_connect(GTK_OBJECT(mbutton), "clicked",
-                           GTK_SIGNAL_FUNC(new_game), NULL);
+	mbutton = gtk_button_new ();
+	g_signal_connect (G_OBJECT (mbutton), "clicked",
+			  G_CALLBACK (new_game), NULL);
         gtk_widget_set_size_request (mbutton, 38, 38);
-	gtk_table_attach(GTK_TABLE(button_table), mbutton, 1, 2, 0, 1,
-			 0, 0, 5, 5);
+	gtk_table_attach (GTK_TABLE (button_table), mbutton, 1, 2, 0, 1,
+			  0, 0, 5, 5);
 
-	face_box = gtk_vbox_new(FALSE, 0);
-	gtk_container_add(GTK_CONTAINER(mbutton), face_box);
+	face_box = gtk_vbox_new (FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (mbutton), face_box);
 	
 	pm_win     = image_widget_setup ("gnomine/face-win.xpm");
 	pm_sad     = image_widget_setup ("gnomine/face-sad.xpm");
@@ -843,88 +899,84 @@ main (int argc, char *argv[])
 	pm_cool    = image_widget_setup ("gnomine/face-cool.xpm");
 	pm_worried = image_widget_setup ("gnomine/face-worried.xpm");
 
-        gtk_box_pack_start(GTK_BOX(face_box), pm_win, FALSE, FALSE, 0);
-        gtk_box_pack_start(GTK_BOX(face_box), pm_sad, FALSE, FALSE, 0);
-        gtk_box_pack_start(GTK_BOX(face_box), pm_smile, FALSE, FALSE, 0);
-        gtk_box_pack_start(GTK_BOX(face_box), pm_cool, FALSE, FALSE, 0);
-        gtk_box_pack_start(GTK_BOX(face_box), pm_worried, FALSE, FALSE, 0);
+        gtk_box_pack_start (GTK_BOX (face_box), pm_win, FALSE, FALSE, 0);
+        gtk_box_pack_start (GTK_BOX (face_box), pm_sad, FALSE, FALSE, 0);
+        gtk_box_pack_start (GTK_BOX (face_box), pm_smile, FALSE, FALSE, 0);
+        gtk_box_pack_start (GTK_BOX (face_box), pm_cool, FALSE, FALSE, 0);
+        gtk_box_pack_start (GTK_BOX (face_box), pm_worried, FALSE, FALSE, 0);
 
-	show_face(pm_smile);
+	show_face (pm_smile); 
 
-	gtk_widget_show(face_box);
-	gtk_widget_show(mbutton);
-
-	gtk_widget_show(button_table);
-	
         align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
 	gtk_table_attach (GTK_TABLE (button_table), align, 1, 2, 1, 2,
 			  GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL,
 			  0, 0);
-        gtk_widget_show (align);
   
-	box = gtk_vbox_new(FALSE, 0);
+	box = gtk_vbox_new (FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (align), box);
-	mfield = gtk_minefield_new();
-	gtk_box_pack_start(GTK_BOX(box), mfield, TRUE, TRUE, 0);
+	mfield = gtk_minefield_new ();
+	gtk_box_pack_start (GTK_BOX (box), mfield, TRUE, TRUE, 0);
 
         ralign = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
-	gtk_box_pack_start (GTK_BOX(box), ralign, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (box), ralign, TRUE, TRUE, 0);
 	rbutton = gtk_button_new_with_label ("Press to resume");
-	g_signal_connect (GTK_OBJECT(rbutton), "clicked", 
-			    GTK_SIGNAL_FUNC (resume_game_cb), NULL);
-        gtk_container_add (GTK_CONTAINER(ralign), rbutton);
-        gtk_widget_show (rbutton);
-	gtk_widget_show (box);
+	g_signal_connect (G_OBJECT (rbutton), "clicked", 
+			  G_CALLBACK (resume_game_cb), NULL);
+        gtk_container_add (GTK_CONTAINER (ralign), rbutton);
 
-        setup_mode(mfield, fsize);
+        setup_mode (mfield, fsize);
+
+	gtk_minefield_set_use_question_marks (GTK_MINEFIELD (mfield),
+					      use_question_marks);
 	
-	g_signal_connect(GTK_OBJECT(mfield), "marks_changed",
-			   GTK_SIGNAL_FUNC(marks_changed), NULL);
-	g_signal_connect(GTK_OBJECT(mfield), "explode",
-			   GTK_SIGNAL_FUNC(lose_game), NULL);
-	g_signal_connect(GTK_OBJECT(mfield), "win",
-			   GTK_SIGNAL_FUNC(win_game), NULL);
-	g_signal_connect(GTK_OBJECT(mfield), "look",
-			   GTK_SIGNAL_FUNC(look_cell), NULL);
-	g_signal_connect(GTK_OBJECT(mfield), "unlook",
-			   GTK_SIGNAL_FUNC(unlook_cell), NULL);
+	g_signal_connect (G_OBJECT (mfield), "marks_changed",
+			  G_CALLBACK (marks_changed), NULL);
+	g_signal_connect (G_OBJECT (mfield), "explode",
+			  G_CALLBACK (lose_game), NULL);
+	g_signal_connect (G_OBJECT (mfield), "win",
+			  G_CALLBACK (win_game), NULL);
+	g_signal_connect (G_OBJECT (mfield), "look",
+			  G_CALLBACK (look_cell), NULL);
+	g_signal_connect (G_OBJECT (mfield), "unlook",
+			  G_CALLBACK (unlook_cell), NULL);
 	
-	gtk_widget_show(mfield);
+	status_box = gtk_hbox_new (FALSE, GNOME_PAD);
+	gtk_box_pack_start (GTK_BOX (appbar), status_box, 
+			    FALSE, FALSE, 0);
 
-        status_table = gtk_table_new(1, 4, TRUE);
-	gtk_box_pack_start(GTK_BOX(all_boxes), status_table, FALSE, TRUE, 0);
 	label = gtk_label_new(_("Flags:"));
-	gtk_table_attach(GTK_TABLE(status_table), label,
-			 0, 1, 0, 1, 0, 0, 3, 3);
-	gtk_widget_show(label);
+	gtk_box_pack_start (GTK_BOX (status_box), label, 
+			    FALSE, FALSE, 0);
 	
-	flabel = gtk_label_new("0");
-	
-	gtk_table_attach(GTK_TABLE(status_table), flabel,
-			 1, 2, 0, 1, 0, 0, 3, 3);
-	gtk_widget_show(flabel);
+	flabel = gtk_label_new ("0");
+	gtk_box_pack_start (GTK_BOX (status_box), flabel, 
+			    FALSE, FALSE, 0);
 
-	label = gtk_label_new(_("Time:"));
-	gtk_table_attach(GTK_TABLE(status_table), label,
-			 2, 3, 0, 1, 0, 0, 3, 3);
-	gtk_widget_show(label);
+	label = gtk_label_new (_("Time:"));
+	gtk_box_pack_start (GTK_BOX (status_box), label, 
+			    FALSE, FALSE, 0);
 
-        clk = games_clock_new();
-	gtk_table_attach(GTK_TABLE(status_table), clk,
-		3, 4, 0, 1, 0, 0, 3 ,3);
-	gtk_widget_show(clk);
-
-        gtk_widget_show(status_table);
-	
-	gtk_widget_show(all_boxes);
+        clk = games_clock_new ();
+	gtk_box_pack_start (GTK_BOX (status_box), clk, 
+			    FALSE, FALSE, 0);
 
 	update_score_state ();
 
-	new_game(mfield, NULL);
+	new_game (mfield, NULL);
 
-        gtk_widget_show(window);
+        gtk_widget_show_all (window);
+	/* All this hiding is a bit ugly, but it's better than a
+	 * ton of gtk_widget_show calls. */
+	gtk_widget_hide (ralign);
+	gtk_widget_hide (pm_win);
+	gtk_widget_hide (pm_sad);
+	gtk_widget_hide (pm_cool);
+	gtk_widget_hide (pm_worried);
 
-        gtk_main();
+	/* shrink the window to recover the space that ralign was using */
+	gtk_window_resize (GTK_WINDOW (window), 1, 1);
+	
+        gtk_main ();
 
 	return 0;
 }
