@@ -1,4 +1,5 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* -*- Mode: C; indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8 -*- */
+
 /*
  * GNOME-Mahjongg
  * (C) 1998-1999 the Free Software Foundation
@@ -26,6 +27,7 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <games-clock.h>
 #include <games-gconf.h>
+#include <games-frame.h>
 
 #include "mahjongg.h"
 #include "solubility.h"
@@ -247,7 +249,6 @@ tile tiles[MAX_TILES];
 GtkWidget *window, *appbar;
 GtkWidget *canvas;
 GtkWidget *tiles_label;
-GtkWidget *seed_label;
 gint selected_tile, visible_tiles;
 gint sequence_number;
 
@@ -292,7 +293,7 @@ gint paused=0;
 
 /* for the preferences */
 GConfClient *conf_client;
-gboolean popup_warn = FALSE, popup_confirm = FALSE;
+gboolean popup_warn = FALSE;
 GtkWidget *warn_cb = NULL, *confirm_cb = NULL;
 GtkWidget *colour_well = NULL;
 GtkWidget *pref_dialog = NULL;
@@ -314,7 +315,7 @@ enum {
 
 static void change_tiles (void);
 static void change_tile_image (tile *tile_inf);
-void clear_undo_queue ();
+static void clear_undo_queue (void);
 void you_won (void);
 void no_match (void);
 void check_free (void);
@@ -347,6 +348,8 @@ GnomeUIInfo gamemenu [] = {
 
 	 GNOMEUIINFO_MENU_RESTART_GAME_ITEM(confirm_action, RESTART_GAME),
 
+	 GNOMEUIINFO_MENU_PAUSE_GAME_ITEM (pause_callback, NULL),
+	 
 	 GNOMEUIINFO_SEPARATOR,
 
 	 GNOMEUIINFO_MENU_UNDO_MOVE_ITEM(undo_tile_callback, NULL),
@@ -369,8 +372,8 @@ GnomeUIInfo gamemenu [] = {
 };
 
 GnomeUIInfo settingsmenu [] = {
-        GNOMEUIINFO_TOGGLEITEM(N_("Show _Tool Bar"),
-			       N_("Toggle display of the toolbar"),
+        GNOMEUIINFO_TOGGLEITEM(N_("_Toolbar"),
+			       N_("Show or hide the toolbar"),
 			       show_tb_callback, NULL),
 
 	GNOMEUIINFO_SEPARATOR,
@@ -407,27 +410,28 @@ GnomeUIInfo toolbar_uiinfo [] = {
 		(gpointer)RESTART_GAME, NULL,
 		GNOME_APP_PIXMAP_STOCK, GTK_STOCK_REFRESH, 0, 0, NULL},
 
-        {GNOME_APP_UI_ITEM, N_("Hint"), N_("Get a hint"), hint_callback,
-		NULL, NULL, GNOME_APP_PIXMAP_STOCK, GTK_STOCK_HELP, GDK_H,
-		GDK_CONTROL_MASK, NULL},
+        /* If you change the place for this button, change the index in
+           the definition of PAUSE_BUTTON below */
+        {GNOME_APP_UI_TOGGLEITEM, N_("Pause"), N_("Pause game"),
+		pause_callback, NULL, NULL,
+		GNOME_APP_PIXMAP_STOCK, GTK_STOCK_STOP, 0, 0, NULL},
 
+	GNOMEUIINFO_SEPARATOR,
+	
         {GNOME_APP_UI_ITEM, N_("Undo"), N_("Undo previous move"),
 		undo_tile_callback, NULL, NULL,
 		GNOME_APP_PIXMAP_STOCK, GTK_STOCK_UNDO, 0, 0, NULL},
 
         {GNOME_APP_UI_ITEM, N_("Redo"), N_("Redo"), redo_tile_callback,
 		NULL, NULL, GNOME_APP_PIXMAP_STOCK, GTK_STOCK_REDO, 0, 0, NULL},
+        {GNOME_APP_UI_ITEM, N_("Hint"), N_("Get a hint"), hint_callback,
+		NULL, NULL, GNOME_APP_PIXMAP_STOCK, GTK_STOCK_HELP, GDK_H,
+		GDK_CONTROL_MASK, NULL},
 
 	{GNOME_APP_UI_ITEM, N_("Shuffle"), N_("Shuffle tiles"),
 		shuffle_tiles_callback, NULL, NULL,
 		GNOME_APP_PIXMAP_STOCK, GTK_STOCK_EXECUTE, 0, 0, NULL},
 
-        /* If you change the place for this button, change the index in
-           the definition of PAUSE_BUTTON below */
-        {GNOME_APP_UI_TOGGLEITEM, N_("Pause"), N_("Pause game"),
-		pause_callback, NULL, NULL,
-		GNOME_APP_PIXMAP_STOCK, GTK_STOCK_STOP, 0, 0, NULL},
-
 #ifdef SOUND_SUPPORT_FINISHED
         {GNOME_APP_UI_TOGGLEITEM, N_("Sound"), N_("Toggle sound"),
 		sound_on_callback, NULL, NULL,
@@ -437,8 +441,44 @@ GnomeUIInfo toolbar_uiinfo [] = {
 	{GNOME_APP_UI_ENDOFINFO}
 };
 
-#define PAUSE_BUTTON GTK_TOGGLE_BUTTON(toolbar_uiinfo[6].widget)
-#define HIGHSCORE_WIDGET gamemenu[9].widget
+#define PAUSE_BUTTON GTK_TOGGLE_BUTTON(toolbar_uiinfo[2].widget)
+#define HIGHSCORE_WIDGET gamemenu[10].widget
+
+/* At the end of the game, hint, shuffle and pause all become unavailable. */
+/* Undo and Redo are handled elsewhere. */
+static void set_menus_sensitive (void)
+{
+	gboolean state;
+	
+	/* Pause */
+	state = game_over != GAME_WON;
+	gtk_widget_set_sensitive (gamemenu[3].widget, state);
+	gtk_widget_set_sensitive (toolbar_uiinfo[2].widget, state);
+
+	/* Hint */
+	state = moves_left > 0;
+	gtk_widget_set_sensitive (gamemenu[7].widget, state);
+	gtk_widget_set_sensitive (toolbar_uiinfo[6].widget, state);
+
+	/* Shuffle */
+	state = game_over != GAME_WON;
+	gtk_widget_set_sensitive (gamemenu[5].widget, state);
+	gtk_widget_set_sensitive (toolbar_uiinfo[6].widget, state);
+
+	/* Restart */
+	state = sequence_number != 1;
+	gtk_widget_set_sensitive (gamemenu[2].widget, state);
+	gtk_widget_set_sensitive (toolbar_uiinfo[1].widget, state);
+}
+
+/* Undo and redo sensitivity functionality. */
+static void set_undoredo_sensitive (gboolean undo, gboolean redo)
+{
+	gtk_widget_set_sensitive(gamemenu[5].widget, undo);
+	gtk_widget_set_sensitive(toolbar_uiinfo[4].widget, undo);
+	gtk_widget_set_sensitive(gamemenu[6].widget, redo);
+	gtk_widget_set_sensitive(toolbar_uiinfo[5].widget, redo);
+}
 
 static void
 tileset_callback (GtkWidget *widget, void *data)
@@ -451,11 +491,11 @@ tileset_callback (GtkWidget *widget, void *data)
 			NULL);
 }
 
-void
+static void
 tileset_changed_cb (GConfClient *client,
-		guint cnxn_id,
-		GConfEntry *entry,
-		gpointer user_data)
+		    guint cnxn_id,
+		    GConfEntry *entry,
+		    gpointer user_data)
 {
 	char *tile_tmp = NULL, *bg_tmp = NULL;
 
@@ -488,11 +528,11 @@ bg_callback (GtkWidget *widget, void *data)
 			NULL);
 }
 
-void
+static void
 bg_changed_cb (GConfClient *client,
-		guint cnxn_id,
-		GConfEntry *entry,
-		gpointer user_data)
+	       guint cnxn_id,
+	       GConfEntry *entry,
+	       gpointer user_data)
 {
 	char *tile_tmp = NULL, *bg_tmp = NULL;
 
@@ -514,7 +554,7 @@ bg_changed_cb (GConfClient *client,
 	//FIXME apply in the GUI
 }
 
-void
+static void
 popup_warn_callback (GtkWidget *widget, gpointer data)
 {
 	popup_warn = gtk_toggle_button_get_active
@@ -524,7 +564,7 @@ popup_warn_callback (GtkWidget *widget, gpointer data)
 			"/apps/mahjongg/warn", popup_warn, NULL);
 }
 
-void
+static void
 popup_warn_changed_cb (GConfClient *client,
 		guint cnxn_id,
 		GConfEntry *entry,
@@ -544,37 +584,7 @@ popup_warn_changed_cb (GConfClient *client,
 	}
 }
 
-void
-popup_confirm_callback (GtkWidget *widget, gpointer data)
-{
-	popup_confirm = gtk_toggle_button_get_active
-		(GTK_TOGGLE_BUTTON (widget));
-
-	gconf_client_set_bool (conf_client,
-			"/apps/mahjongg/confirm", popup_confirm, NULL);
-}
-
-void
-popup_confirm_changed_cb (GConfClient *client,
-		guint cnxn_id,
-		GConfEntry *entry,
-		gpointer user_data)
-{
-	gboolean popup_confirm_tmp;
-
-	popup_confirm_tmp = gconf_client_get_bool (conf_client,
-			"/apps/mahjongg/confirm", NULL);
-	if (popup_confirm_tmp != popup_confirm)
-	{
-		popup_confirm = popup_confirm_tmp;
-		if (confirm_cb != NULL)
-			gtk_toggle_button_set_active
-				(GTK_TOGGLE_BUTTON (confirm_cb),
-				popup_confirm);
-	}
-}
-
-void
+static void
 show_toolbar_changed_cb (GConfClient *client,
 		guint cnxn_id,
 		GConfEntry *entry,
@@ -584,7 +594,7 @@ show_toolbar_changed_cb (GConfClient *client,
 	gboolean shown;
 
 	shown = gconf_client_get_bool (conf_client,
-			"/apps/mahjongg/show-toolbar", NULL);
+			"/apps/mahjongg/show_toolbar", NULL);
 
 	gdi = gnome_app_get_dock_item_by_name (GNOME_APP (window),
 			GNOME_APP_TOOLBAR_NAME);
@@ -607,14 +617,14 @@ show_tb_callback (GtkWidget *widget, gpointer data)
 {
 	if ((GTK_CHECK_MENU_ITEM (widget))->active) {
 		gconf_client_set_bool(conf_client,
-				"/apps/mahjongg/show-toolbar", TRUE, NULL);
+				"/apps/mahjongg/show_toolbar", TRUE, NULL);
 	} else {
 		gconf_client_set_bool(conf_client,
-				"/apps/mahjongg/show-toolbar", FALSE, NULL);
+				"/apps/mahjongg/show_toolbar", FALSE, NULL);
 	}
 }
 
-void
+static void
 bg_colour_changed_cb (GConfClient *client,
 		      guint cnxn_id,
 		      GConfEntry *entry,
@@ -635,7 +645,7 @@ bg_colour_changed_cb (GConfClient *client,
 	}
 }
 
-void
+static void
 bg_colour_callback (GtkWidget *widget, gpointer data)
 {
 	static char *tmp = "";
@@ -649,7 +659,7 @@ bg_colour_callback (GtkWidget *widget, gpointer data)
 			"/apps/mahjongg/bgcolour", tmp, NULL);
 }
 
-void
+static void
 mapset_changed_cb (GConfClient *client,
 		   guint        cnxn_id,
 		   GConfEntry  *entry,
@@ -674,12 +684,12 @@ mapset_changed_cb (GConfClient *client,
 		GTK_BUTTONS_OK,
 		_("This new mapset will take effect when you start "
 		  "a new game, or when Mahjongg is restarted."));
-
+	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 	gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 }
 
-void
+static void
 set_map_selection (GtkWidget *widget, void *data)
 {
 	struct _maps *map = (struct _maps*) data;
@@ -692,14 +702,14 @@ set_map_selection (GtkWidget *widget, void *data)
 				 mapset, NULL);
 }
 
-void
+static void
 init_config (void)
 {
 	gconf_client_add_dir (conf_client,
 			"/apps/mahjongg", GCONF_CLIENT_PRELOAD_ONELEVEL,
 			NULL);
 	gconf_client_notify_add (conf_client,
-			"/apps/mahjongg/show-toolbar",
+			"/apps/mahjongg/show_toolbar",
 			show_toolbar_changed_cb,
 			NULL, NULL, NULL);
 	gconf_client_notify_add (conf_client,
@@ -710,10 +720,6 @@ init_config (void)
 			"/apps/mahjongg/mapset",
 			mapset_changed_cb,
 			NULL, NULL, NULL);
-	gconf_client_notify_add (conf_client,
-			"/apps/mahjongg/confirm",
-			popup_confirm_changed_cb,
-			NULL, NULL, NULL);
 	gconf_client_notify_add (conf_client,
 			"/apps/mahjongg/warn",
 			popup_warn_changed_cb,
@@ -734,33 +740,31 @@ free_str (GtkWidget *widget, void *data)
 	g_free (data);
 }
 
-void
+static void
 message (gchar *message)
 {
 	gnome_appbar_pop (GNOME_APPBAR (appbar));
 	gnome_appbar_push (GNOME_APPBAR (appbar), message);
 }
 
-void update_score_state ()
+static void 
+update_score_state ()
 {
         gchar **names = NULL;
         gfloat *scores = NULL;
         time_t *scoretimes = NULL;
 	gint top;
 
-	top = gnome_score_get_notable(APPNAME, score_current_mapset, &names, &scores, &scoretimes);
-	if (top > 0) {
-		gtk_widget_set_sensitive (HIGHSCORE_WIDGET, TRUE);
-		g_strfreev(names);
-		g_free(scores);
-		g_free(scoretimes);
-	} else {
-		gtk_widget_set_sensitive (HIGHSCORE_WIDGET, FALSE);
-	}
+	top = gnome_score_get_notable (APPNAME, score_current_mapset,
+				       &names, &scores, &scoretimes);
+	gtk_widget_set_sensitive (HIGHSCORE_WIDGET, top > 0);
+	g_strfreev (names);
+	g_free (scores);
+	g_free (scoretimes);
 }
 
 
-void
+static void
 chrono_start (void)
 {
 	games_clock_stop (GAMES_CLOCK (chrono));
@@ -768,7 +772,7 @@ chrono_start (void)
 	games_clock_start (GAMES_CLOCK (chrono));
 }
 
-gint
+static gint
 update_moves_left (void)
 {
         char *tmpstr;
@@ -795,7 +799,8 @@ set_backgnd_colour (gchar *str)
 	colourmap = gtk_widget_get_colormap (canvas);
 	gdk_color_parse (backgnd.name, &backgnd.colour);
 
-	gdk_color_alloc (colourmap, &backgnd.colour);
+	gdk_colormap_alloc_color (colourmap, &backgnd.colour,
+				  FALSE, TRUE);
 
 	widget_style = gtk_widget_get_style (canvas);
 	temp_style = gtk_style_copy (widget_style);
@@ -805,7 +810,7 @@ set_backgnd_colour (gchar *str)
 	temp_style->bg[3] = backgnd.colour;
 	temp_style->bg[4] = backgnd.colour;
 	gtk_widget_set_style (canvas, temp_style);
-	gnome_canvas_update_now (GNOME_CANVAS(canvas));
+	gnome_canvas_update_now (GNOME_CANVAS (canvas));
 }
 
 static void
@@ -847,15 +852,15 @@ change_tile_image (tile *tile_inf)
 			tile_inf->current_image, NULL);
 }
 
-void
+static void
 select_tile (tile *tile_inf)
 {
         tile_inf->selected |= SELECTED_FLAG;
-        change_tile_image(tile_inf);
+        change_tile_image (tile_inf);
         selected_tile = tile_inf->number;
 }
 
-void
+static void
 unselect_tile (tile *tile_inf)
 {
         selected_tile = MAX_TILES + 1;
@@ -895,6 +900,8 @@ tile_event (GnomeCanvasItem *item, GdkEvent *event, tile *tile_inf)
 							tmpstr = g_strdup_printf("%3d", visible_tiles);
 							gtk_label_set_text (GTK_LABEL(tiles_label), tmpstr);
 							update_moves_left ();
+							set_undoredo_sensitive (TRUE, FALSE);
+							set_menus_sensitive ();
 
 							if (visible_tiles <= 0) {
 								games_clock_stop(GAMES_CLOCK(chrono));
@@ -1020,6 +1027,7 @@ no_match (void)
 				GTK_MESSAGE_INFO,
 				GTK_BUTTONS_OK,
 				_("These tiles don't match."));
+		gtk_dialog_set_has_separator (GTK_DIALOG (mb), FALSE);
 		gtk_dialog_run (GTK_DIALOG (mb));
 		gtk_widget_destroy (mb);
         } else
@@ -1049,6 +1057,7 @@ check_free (void)
  	if ((moves_left == 0) && (visible_tiles>0)) { 
                 GtkWidget *mb;
 
+		set_menus_sensitive ();
                 if (!game_over) {
 			mb = gtk_message_dialog_new (GTK_WINDOW (window),
 						     GTK_DIALOG_MODAL
@@ -1056,6 +1065,7 @@ check_free (void)
 						     GTK_MESSAGE_INFO,
 						     GTK_BUTTONS_OK,
 						     (_("There are no more moves.")));
+			gtk_dialog_set_has_separator (GTK_DIALOG (mb), FALSE);
 			gtk_dialog_run (GTK_DIALOG (mb));
 			gtk_widget_destroy (mb);
                 }
@@ -1076,8 +1086,11 @@ you_won (void)
         seconds = GAMES_CLOCK (chrono)->stopped;
 
         score = (seconds / 60) * 1.0 + (seconds % 60) / 100.0;
-        if (pos = gnome_score_log (score, score_current_mapset, FALSE)) {
-                dialog = gnome_scores_display (_(APPNAME_LONG), APPNAME, score_current_mapset, pos);
+	pos = gnome_score_log (score, score_current_mapset, FALSE);
+	set_menus_sensitive ();
+        if (pos) {
+                dialog = gnome_scores_display (_(APPNAME_LONG), APPNAME,
+					       score_current_mapset, pos);
 		if (dialog != NULL) {
 			gtk_window_set_transient_for (GTK_WINDOW(dialog),
 						      GTK_WINDOW(window));
@@ -1096,27 +1109,13 @@ pref_dialog_response (GtkDialog *dialog, gint response, gpointer data)
 	colour_well = NULL;
 }
 
-GtkWidget *
-bold_frame (gchar * title)
-{
-	gchar *markup;
-	GtkWidget * frame;
-	
-	markup = g_strdup_printf ("<b>%s</b>",title);
-	frame = gtk_frame_new(markup);
-	g_free(markup);
-	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
-	gtk_label_set_use_markup (GTK_LABEL (gtk_frame_get_label_widget(GTK_FRAME(frame))), TRUE);
-	
-	return frame;
-}
-
 void
 properties_callback (GtkWidget *widget, gpointer data)
 {
 	GtkWidget *menu, *omenu;
-	GtkWidget *vbox, *frame, *table, *w, *label;
+	GtkWidget *frame, *table, *w, *label;
 	GtkSizeGroup *group;
+	GtkWidget *top_table;
 
 	if (pref_dialog) {
 		gtk_window_present (GTK_WINDOW (pref_dialog));
@@ -1128,23 +1127,26 @@ properties_callback (GtkWidget *widget, gpointer data)
 			GTK_DIALOG_DESTROY_WITH_PARENT,
 			GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 			NULL);
-
+	gtk_dialog_set_has_separator (GTK_DIALOG (pref_dialog), FALSE);
 	gtk_window_set_resizable (GTK_WINDOW (pref_dialog), FALSE);
 	gtk_dialog_set_default_response (GTK_DIALOG (pref_dialog),
 					 GTK_RESPONSE_CLOSE);
 	g_signal_connect (G_OBJECT (pref_dialog), "response",
-			  G_CALLBACK(pref_dialog_response), NULL);
+			  G_CALLBACK (pref_dialog_response), NULL);
 
 	group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
-	vbox = gtk_vbox_new (FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 8);
+	top_table = gtk_table_new (2, 2, FALSE);
+	gtk_container_set_border_width (GTK_CONTAINER (top_table), 0);
+	gtk_table_set_row_spacings (GTK_TABLE (top_table), 0);
+	gtk_table_set_col_spacings (GTK_TABLE (top_table), 0);
 
-	frame = bold_frame (_("Tiles"));
+	frame = games_frame_new (_("Tiles"));
+	gtk_table_attach_defaults (GTK_TABLE (top_table), frame, 0, 1, 0, 1);
 
 	table = gtk_table_new (2, 2, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (table), 8);
-	gtk_table_set_row_spacings (GTK_TABLE (table), 4);
+	gtk_container_set_border_width (GTK_CONTAINER (table), 6);
+	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
 	gtk_table_set_col_spacings (GTK_TABLE (table), 8);
 	
 	label = gtk_label_new (_("Tile Set"));
@@ -1155,7 +1157,7 @@ properties_callback (GtkWidget *widget, gpointer data)
 	omenu = gtk_option_menu_new ();
 	menu = gtk_menu_new ();
 	fill_tile_menu (menu, "mahjongg", 1);
-	gtk_option_menu_set_menu (GTK_OPTION_MENU(omenu), menu);
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu), menu);
 	gtk_table_attach_defaults (GTK_TABLE (table), omenu, 1, 2, 0, 1);
 
 	label = gtk_label_new (_("Tile Background"));
@@ -1166,18 +1168,17 @@ properties_callback (GtkWidget *widget, gpointer data)
 	omenu = gtk_option_menu_new ();
 	menu = gtk_menu_new ();
 	fill_tile_menu (menu, "mahjongg/bg", 0);
-	gtk_option_menu_set_menu (GTK_OPTION_MENU(omenu), menu);
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu), menu);
 	gtk_table_attach_defaults (GTK_TABLE (table), omenu, 1, 2, 1, 2);
 
 	gtk_container_add(GTK_CONTAINER (frame), table);
-	gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);
 
-
-	frame = bold_frame (_("Maps"));
+	frame = games_frame_new (_("Maps"));
+	gtk_table_attach_defaults (GTK_TABLE (top_table), frame, 0, 1, 1, 2);
 
 	table = gtk_table_new (1, 2, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (table), 8);
-	gtk_table_set_row_spacings (GTK_TABLE (table), 4);
+	gtk_container_set_border_width (GTK_CONTAINER (table), 6);
+	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
 	gtk_table_set_col_spacings (GTK_TABLE (table), 8);
 	
 	label = gtk_label_new (_("Select Map"));
@@ -1188,18 +1189,17 @@ properties_callback (GtkWidget *widget, gpointer data)
 	omenu = gtk_option_menu_new ();
 	menu = gtk_menu_new ();
 	fill_map_menu (menu);
-	gtk_option_menu_set_menu (GTK_OPTION_MENU(omenu), menu);
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu), menu);
 	gtk_table_attach_defaults (GTK_TABLE (table), omenu, 1, 2, 0, 1);
 
 	gtk_container_add(GTK_CONTAINER (frame), table);
-	gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);
 
-
-	frame = bold_frame (_("Colours"));
+	frame = games_frame_new (_("Colors"));
+	gtk_table_attach_defaults (GTK_TABLE (top_table), frame, 1, 2, 0, 1);
 
 	table = gtk_table_new (1, 2, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (table), 8);
-	gtk_table_set_row_spacings (GTK_TABLE (table), 4);
+	gtk_container_set_border_width (GTK_CONTAINER (table), 6);
+	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
 	gtk_table_set_col_spacings (GTK_TABLE (table), 8);
 	
 	label = gtk_label_new (_("Background Color"));
@@ -1208,45 +1208,39 @@ properties_callback (GtkWidget *widget, gpointer data)
 	gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);
 
 	{
-	  int ur,ug,ub ;
+	  int ur, ug, ub;
 
-	  w  = gnome_color_picker_new();
-	  sscanf (backgnd.name, "#%02x%02x%02x", &ur,&ug,&ub);
-	  gnome_color_picker_set_i8 (GNOME_COLOR_PICKER(w), ur, ug, ub, 0);
-	  g_signal_connect (G_OBJECT(w), "color_set",
+	  w  = gnome_color_picker_new ();
+	  sscanf (backgnd.name, "#%02x%02x%02x", &ur, &ug, &ub);
+	  gnome_color_picker_set_i8 (GNOME_COLOR_PICKER (w), ur, ug, ub, 0);
+	  g_signal_connect (G_OBJECT (w), "color_set",
 			    G_CALLBACK (bg_colour_callback), &backgnd.name);
 	}
 	gtk_table_attach_defaults (GTK_TABLE (table), w, 1, 2, 0, 1);
 
-	gtk_container_add(GTK_CONTAINER (frame), table);
-	gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);
+	gtk_container_add (GTK_CONTAINER (frame), table);
 
 
-	frame = bold_frame (_("Warnings"));
+	frame = games_frame_new (_("Warnings"));
+	gtk_table_attach_defaults (GTK_TABLE (top_table), frame, 1, 2, 1, 2);
 
-	table = gtk_vbox_new (FALSE, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (table), 8);
+	table = gtk_vbox_new (FALSE, 6);
+	gtk_container_set_border_width (GTK_CONTAINER (table), 6);
 
 	w = gtk_check_button_new_with_label (_("Warn when tiles don't match"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), popup_warn);
 	g_signal_connect (G_OBJECT(w), "clicked", G_CALLBACK (popup_warn_callback), NULL);
 	gtk_box_pack_start_defaults (GTK_BOX (table), w);
-
-	w = gtk_check_button_new_with_label (_("Show confirmation dialogs"));
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), popup_confirm);
-	g_signal_connect (G_OBJECT(w), "clicked", G_CALLBACK (popup_confirm_callback), NULL);
-	gtk_box_pack_start_defaults (GTK_BOX(table), w);
 	
 	gtk_container_add(GTK_CONTAINER (frame), table);
-	gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);
 
-	gtk_box_pack_start_defaults (GTK_BOX(GTK_DIALOG(pref_dialog)->vbox), vbox);
+	gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (pref_dialog)->vbox), top_table);
 
 	g_object_unref (group);
 	gtk_widget_show_all (pref_dialog);
 }
 
-gint
+static gint
 hint_timeout (gpointer data)
 {
 	timeout_counter ++;
@@ -1320,7 +1314,8 @@ hint_callback (GtkWidget *widget, gpointer data)
 	games_clock_start (GAMES_CLOCK(chrono));
 }
 
-void about_destroy (GtkWidget * widget, gpointer data)
+static void
+about_destroy (GtkWidget * widget, gpointer data)
 {
 	about = NULL;
 }
@@ -1330,13 +1325,16 @@ about_callback (GtkWidget *widget, gpointer data)
 {
 	GdkPixbuf *pixbuf = NULL;
 	const gchar *authors [] = {
-		"Code: Francisco Bustamante",
-		"      Max Watson",
-		"      Heinz Hempe",
-		"      Michael Meeks",
-                "      Philippe Chavin",
-		"Tiles: Jonathan Buzzard",
-		"       Max Watson",
+		N_("Main game:"),
+		"Francisco Bustamante",
+		"Max Watson",
+		"Heinz Hempe",
+		"Michael Meeks",
+                "Philippe Chavin",
+		"",
+		N_("Tiles:"),
+		"Jonathan Buzzard",
+		"Max Watson",
 		NULL
 	};
 	gchar *documenters[] = {
@@ -1364,11 +1362,10 @@ about_callback (GtkWidget *widget, gpointer data)
 	}
 	
 	about = gnome_about_new (_("GNOME Mahjongg"), VERSION,
-				 "(C) 1998 The Free Software Foundation",
-				  _("Send comments and bug reports to:\n"
-				   "        pancho@nuclecu.unam.mx or\n"
-				   "        mmeeks@gnu.org\n\n"
-				   "Tiles under the General Public License."),
+				 "Copyright \xc2\xa9 1998-2003 Free Software "
+				 "Foundation, Inc.",
+				 _("GNOME version of the Eastern tile game, "
+				   "Mahjongg."),
 				 (const gchar **)authors,
 				 (const gchar **)documenters,
 				 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
@@ -1388,10 +1385,6 @@ void
 pause_callback (void)
 {
         gint i;
-        if (game_over) {
-                gtk_toggle_button_set_active (PAUSE_BUTTON, FALSE);
-                return;
-        }
         paused = !paused;
         if (paused) {
                 games_clock_stop (GAMES_CLOCK (chrono));
@@ -1436,7 +1429,7 @@ scores_callback (GtkWidget *widget, gpointer data)
 	}
 }
 
-void
+static void
 init_game (void)
 {
         gtk_label_set_text (GTK_LABEL (tiles_label), MAX_TILES_STR);
@@ -1446,6 +1439,8 @@ init_game (void)
         visible_tiles = MAX_TILES;
         selected_tile = MAX_TILES + 1;
         gnome_canvas_update_now(GNOME_CANVAS(canvas));
+	set_undoredo_sensitive (FALSE, FALSE);
+	set_menus_sensitive ();
 
         chrono_start();
 }
@@ -1453,78 +1448,20 @@ init_game (void)
 void
 confirm_action (GtkWidget *widget, gpointer data)
 {
-	gboolean doit = TRUE;
-
-	if (popup_confirm == TRUE) {
-		gchar *confirm_text;
-		GtkWidget *dialog;
-		gint response;
-		
-		switch ((game_state)data) {
-		case RESTART_GAME :
-			confirm_text = _("Are you sure you want to restart this game?");
-			break;
-		case QUIT_GAME :
-			/* GNOME IS AN ACRONYM, DAMNIT! */
-			confirm_text = _("Are you sure you want to quit GNOME Mahjongg?");
-			break;
-		case NEW_GAME:
-		case NEW_GAME_WITH_SEED:
-			confirm_text = _("Are you sure you want to start a new game?");
-			break;
-		default:
-			confirm_text = _("Serious internal error");
-			break;
-		}
-
-		/* Special case the quit because the buttons are different */
-
-		if ((game_state)data == QUIT_GAME)
-		{
-			dialog = gtk_message_dialog_new (GTK_WINDOW (window),
-						 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-						 GTK_MESSAGE_QUESTION,
-						 GTK_BUTTONS_NONE,
-						 confirm_text);
-			gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-						GTK_STOCK_CANCEL,
-						GTK_RESPONSE_NO,
-						GTK_STOCK_QUIT,
-						GTK_RESPONSE_YES,
-						NULL);
-		}
-		else {
-                        dialog = gtk_message_dialog_new (GTK_WINDOW (window),
-                                                 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                                 GTK_MESSAGE_QUESTION,
-                                                 GTK_BUTTONS_YES_NO,
-                                                 confirm_text);
-		}
-
-		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_YES);
-
-		response = gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (dialog);
-		
-		doit = (response == GTK_RESPONSE_YES);
-	}
-	
-	if (doit) {
-		switch ((gint)data) {
-		case NEW_GAME:
-		case NEW_GAME_WITH_SEED:
-			ensure_pause_off ();
-			new_game ((gint)data == NEW_GAME);
-			break;
-		case RESTART_GAME:
-			restart_game ();
-			break;
-		case QUIT_GAME:
-			gtk_main_quit ();
-			break;
-		default:
-			break;
-		}
+	switch ((gint)data) {
+	case NEW_GAME:
+	case NEW_GAME_WITH_SEED:
+		ensure_pause_off ();
+		new_game ((gint)data == NEW_GAME);
+		break;
+	case RESTART_GAME:
+		restart_game ();
+		break;
+	case QUIT_GAME:
+		gtk_main_quit ();
+		break;
+	default:
+		break;
 	}
 }
 
@@ -1586,7 +1523,13 @@ redo_tile_callback (GtkWidget *widget, gpointer data)
                   	gnome_app_flash (GNOME_APP (window), "No more redo!");
         tmpstr = g_strdup_printf ("%3d",visible_tiles);
         gtk_label_set_text(GTK_LABEL (tiles_label), tmpstr);
-        
+
+        set_undoredo_sensitive (TRUE, FALSE);
+	for (i=0; i<MAX_TILES; i++) {
+		if (tiles[i].sequence == sequence_number)
+			set_undoredo_sensitive (TRUE, TRUE);
+	}
+	
         update_moves_left ();
         gnome_canvas_update_now (GNOME_CANVAS (canvas));
 }
@@ -1624,6 +1567,8 @@ undo_tile_callback (GtkWidget *widget, gpointer data)
         gtk_label_set_text (GTK_LABEL(tiles_label), tmpstr);
         gnome_canvas_update_now (GNOME_CANVAS (canvas));
 
+	set_menus_sensitive ();
+	set_undoredo_sensitive (sequence_number>1, TRUE);
         update_moves_left ();
 }
 
@@ -1642,7 +1587,7 @@ select_game (GtkWidget *widget, gpointer data)
 						 GTK_STOCK_OK,
 						 GTK_RESPONSE_OK,
 						 NULL);
-	
+	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 	box = GTK_DIALOG (dialog)->vbox;
@@ -1681,7 +1626,7 @@ sound_on_callback (GtkWidget *widget, gpointer data)
 }
 
 /* You loose your re-do queue when you make a move */
-void
+static void
 clear_undo_queue (void)
 {
 	gint lp ;
@@ -1691,7 +1636,7 @@ clear_undo_queue (void)
 			tiles[lp].sequence = 0 ;
 }
 
-void
+static void
 load_map (void)
 {
 	gchar* name = mapset;
@@ -1716,36 +1661,38 @@ load_map (void)
 		}
 }
 
-gint
+static gint
 canvas_x (gint i)
 {
-	return pos[i].x * (HALF_WIDTH-0) + xpos_offset + (THICKNESS * pos[i].layer);
+	return pos[i].x * (HALF_WIDTH-0) + xpos_offset
+		+ (THICKNESS * pos[i].layer);
 }
 
-gint
+static gint
 canvas_y (gint i)
 {
-	return pos[i].y * (HALF_HEIGHT-0) + ypos_offset - (THICKNESS * pos[i].layer);
+	return pos[i].y * (HALF_HEIGHT-0) + ypos_offset
+		- (THICKNESS * pos[i].layer);
 }
 
-void
+static void
 load_images (void)
 {
 	gint i;
   
 	for (i = MAX_TILES - 1; i >= 0; i --) {
 		gnome_canvas_item_set (tiles[i].image_item,
-				       "x", (double)canvas_x(i),
-				       "y", (double)canvas_y(i),
+				       "x", (double)canvas_x (i),
+				       "y", (double)canvas_y (i),
 				       NULL);
 		gnome_canvas_item_set (tiles[i].bg_item,
-				       "x", (double)canvas_x(i),
-				       "y", (double)canvas_y(i),
+				       "x", (double)canvas_x (i),
+				       "y", (double)canvas_y (i),
 				       NULL);
 	}
 }
 
-void
+static void
 create_canvas_items (void)
 {
 	gint orig_x, orig_y, i;
@@ -1803,14 +1750,16 @@ load_tiles (gchar *fname, gchar *bg_fname)
 {
 	gchar *tmp, *fn, *bg_fn;
 
-	tmp = g_strconcat ("mahjongg/", fname, NULL);
+	tmp = g_build_filename ("mahjongg", fname, NULL);
 	
-	fn = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP, (tmp), FALSE, NULL);
+	fn = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP,
+					(tmp), FALSE, NULL);
 	g_free (tmp);
 
-	tmp = g_strconcat ("mahjongg/bg/", bg_fname, NULL);
+	tmp = g_build_filename ("mahjongg", "bg", bg_fname, NULL);
 
-	bg_fn = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP, (tmp), FALSE, NULL);
+	bg_fn = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP,
+					   (tmp), FALSE, NULL);
 	g_free (tmp);
 
 	if (!g_file_test ((fn), G_FILE_TEST_EXISTS)) {
@@ -1822,7 +1771,7 @@ load_tiles (gchar *fname, gchar *bg_fname)
 					      GTK_MESSAGE_ERROR,
 					      GTK_BUTTONS_OK,
 					      s);
-	
+		gtk_dialog_set_has_separator (GTK_DIALOG (box), FALSE);
 		gtk_dialog_run (GTK_DIALOG (box));
 		
 		exit (1);
@@ -1837,6 +1786,7 @@ load_tiles (gchar *fname, gchar *bg_fname)
 					      GTK_MESSAGE_ERROR,
 					      GTK_BUTTONS_OK,
 					      s);
+		gtk_dialog_set_has_separator (GTK_DIALOG (box), FALSE);
 		gtk_dialog_run (GTK_DIALOG (box));
 
 		exit (1);
@@ -1868,8 +1818,8 @@ do_game (void)
 	char *str;
 
 	current_seed = next_seed;
-	str = g_strdup_printf ("%d", current_seed);
-	gtk_label_set_text (GTK_LABEL (seed_label), str);
+	str = g_strdup_printf ("%s (%d)",_("Mahjongg"), current_seed);
+	gtk_window_set_title (GTK_WINDOW (window), str);
 	g_free (str);
 	
 	load_map (); /* assigns pos, and calculates dependencies */
@@ -1885,7 +1835,8 @@ create_mahjongg_board (GtkWidget *mbox)
 
 	gtk_box_pack_start_defaults (GTK_BOX (mbox), canvas);
 
-	gtk_widget_set_usize (canvas, AREA_WIDTH, AREA_HEIGHT);
+	gtk_widget_set_size_request (GTK_WIDGET (canvas),
+				     AREA_WIDTH, AREA_HEIGHT);
 	gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (canvas), 1);
 	gnome_canvas_set_scroll_region (GNOME_CANVAS (canvas),
 					0, 0, AREA_WIDTH, AREA_HEIGHT);
@@ -1911,9 +1862,6 @@ create_mahjongg_board (GtkWidget *mbox)
 	popup_warn = gconf_client_get_bool (conf_client,
 					    "/apps/mahjongg/warn", NULL);
 	
-	popup_confirm = gconf_client_get_bool (conf_client,
-					       "/apps/mahjongg/confirm", NULL);
-
 	do_game ();
 	load_tiles (buf, buf2);
 
@@ -1925,7 +1873,7 @@ create_mahjongg_board (GtkWidget *mbox)
 	update_score_state ();
 }
 
-void
+static void
 new_seed ()
 {
 	struct timeval t;
@@ -1996,6 +1944,7 @@ shuffle_tiles_callback (GtkWidget *widget, gpointer data)
 					     GTK_MESSAGE_ERROR,
 					     GTK_BUTTONS_OK,
 					     (_("Sorry, I was unable to find a playable configuration.")));
+		gtk_dialog_set_has_separator (GTK_DIALOG (mb), FALSE);
 		gtk_dialog_run (GTK_DIALOG (mb));
 		gtk_widget_destroy (mb);
 
@@ -2017,6 +1966,8 @@ shuffle_tiles_callback (GtkWidget *widget, gpointer data)
                 games_clock_set_seconds(GAMES_CLOCK(chrono), (gint) (seconds+60));
                 games_clock_start (GAMES_CLOCK(chrono));
         }
+	
+	set_menus_sensitive ();
 }
 
 int
@@ -2026,6 +1977,7 @@ main (int argc, char *argv [])
 	GtkWidget *chrono_label;
 	GtkWidget *status_box;
 	GtkWidget *group_box;
+	GtkWidget *spacer;
 	gboolean show=TRUE;
 
 	gnome_score_init (APPNAME);
@@ -2056,33 +2008,32 @@ main (int argc, char *argv [])
 	status_box = gtk_hbox_new (FALSE, 10);
 
 	group_box = gtk_hbox_new (FALSE, 0);
-	tiles_label = gtk_label_new (_("Tiles Left: "));
+	tiles_label = gtk_label_new (_("Tiles Left:"));
 	gtk_box_pack_start (GTK_BOX (group_box), tiles_label, FALSE, FALSE, 0);
+	spacer = gtk_label_new (" ");
+	gtk_box_pack_start (GTK_BOX (group_box), spacer, FALSE, FALSE, 0);
 	tiles_label = gtk_label_new (MAX_TILES_STR);
 	gtk_box_pack_start (GTK_BOX (group_box), tiles_label, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (status_box), group_box, FALSE, FALSE, 0);
 
 	group_box = gtk_hbox_new (FALSE, 0);
-	moves_label = gtk_label_new(_("Moves Left: "));
+	moves_label = gtk_label_new(_("Moves Left:"));
 	gtk_box_pack_start (GTK_BOX (group_box), moves_label, FALSE, FALSE, 0);
+	spacer = gtk_label_new (" ");
+	gtk_box_pack_start (GTK_BOX (group_box), spacer, FALSE, FALSE, 0);
 	moves_label = gtk_label_new (MAX_TILES_STR);
 	gtk_box_pack_start (GTK_BOX (group_box), moves_label, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (status_box), group_box, FALSE, FALSE, 0);
 
 	group_box = gtk_hbox_new (FALSE, 0);
-	chrono_label = gtk_label_new (_("Time: "));
+	chrono_label = gtk_label_new (_("Time:"));
 	gtk_box_pack_start (GTK_BOX (group_box), chrono_label, FALSE, FALSE, 0);
+	spacer = gtk_label_new (" ");
+	gtk_box_pack_start (GTK_BOX (group_box), spacer, FALSE, FALSE, 0);
 	chrono = games_clock_new ();
 	gtk_box_pack_start (GTK_BOX (group_box), chrono, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (status_box), group_box, FALSE, FALSE, 0);
 
-	group_box = gtk_hbox_new (FALSE, 0);
-	seed_label = gtk_label_new (_("Seed: "));
-	gtk_box_pack_start (GTK_BOX (group_box), seed_label, FALSE, FALSE, 0);
-	seed_label = gtk_label_new ("");
-	gtk_box_pack_start (GTK_BOX (group_box), seed_label, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (status_box), group_box, FALSE, FALSE, 0);
-
 	/* show the status bar items */
 	gtk_widget_show_all (status_box);
 
@@ -2105,7 +2056,7 @@ main (int argc, char *argv [])
 	gtk_widget_show (window);
 
 	show = gconf_client_get_bool (conf_client,
-			"/apps/mahjongg/show-toolbar", NULL);
+			"/apps/mahjongg/show_toolbar", NULL);
 
 	if (show) {
 		gtk_check_menu_item_set_active
@@ -2121,7 +2072,7 @@ main (int argc, char *argv [])
 		gtk_widget_queue_resize (window);
 	}
 
-	init_config();
+	init_config ();
 
 	score_current_mapset = strdup (mapset);
 	update_score_state ();
