@@ -1,3 +1,5 @@
+/* -*- Mode: C; indent-tabs-mode: nil; tab-width: 8; c-basic-offset: 2 -*- */
+
 /*
  * Gnome-Mahjonggg-solubility algorithem
  * (C) 1998 the Free Software Foundation
@@ -19,6 +21,9 @@
 #include "mahjongg.h"
 #include "solubility.h"
 
+/* Undefine to test the new code. */
+#define USE_CREEPING_HORROR
+
 jmp_buf unsolvable ;
 
 /* If defined this cooks the sequence no.s,
@@ -111,6 +116,20 @@ typeinfo type_info [MAX_TILES/2] = {
 	{ 35, 0, {40, 41} }
 };
 
+#ifndef USE_CREEPING_HORROR
+typedef struct _dep_entry {
+  gboolean free;    /* Is this a valid place to put a tile. */
+  gboolean filled;  /* Has a tile been placed here. */
+  gint foundation[4]; /* Up to four we build on. */
+  gint left[2];  /* Up to two on the left. */
+  gint right[2]; /* and two on the right. */
+  gint overhead[4]; /* Up to four we support. */
+} dep_entry;
+
+dep_entry dependencies[MAX_TILES];
+
+#endif
+
 struct _dep_tree {
   int turn_dep[4] ;	/* Turning dependancies all must be clear */
   int place_dep[4] ;	/* Placing dependancies all must be full */
@@ -205,7 +224,8 @@ void keyPress()
 #endif
 
 /* Takes the last tile places, and avoids its dependancies. */
-int random_free (int lt)
+static int
+random_free (int lt)
 {
   int t, lp, catch ;
   int goodplace ;
@@ -243,7 +263,8 @@ int random_free (int lt)
 }
 
 /* Finds a random block on a layer */
-int random_on_layer (int l)
+static int 
+random_on_layer (int l)
 {
   int t, cnt = 0 ;
   do
@@ -263,7 +284,8 @@ int random_on_layer (int l)
    it returns -1 if there is no tile at that position.
    N.B. Each tile occupies 2x2 positions.
  */
-int tile_at (int layer, int x, int y)
+static int
+tile_at (int layer, int x, int y)
 {
   int lp ;
 
@@ -276,7 +298,8 @@ int tile_at (int layer, int x, int y)
 }
 
 /* This determines whether it is OK to mark this tile as free */
-int ok_free_validate_line (int t)
+static int 
+ok_free_validate_line (int t)
 {
   int i, li, w ;
   int in, out ;
@@ -338,7 +361,8 @@ int ok_free_validate_line (int t)
 /* This simply examines whether the tile should be free or not.
    It does this by examining its dependancies to see if they are
    filled */
-void validate_tile (int t)
+static void
+validate_tile (int t)
 {
   int lfilled, rfilled, lp, free, i, valid ;
 
@@ -400,8 +424,11 @@ void validate_tile (int t)
   if ((lfilled) || (rfilled))
     dep_tree[t].free = 1 ;
 }
+
+#ifdef USE_CREEPING_HORROR
 /* Place tile in map at position f, with pic & type from t */
-void place_tile (int f, int t, int idx)
+static void
+place_tile (int f, int t, int idx)
 {
 #ifdef PLACE_DEBUG
   printf ("Placing at\n") ;
@@ -416,7 +443,7 @@ void place_tile (int f, int t, int idx)
 
   tiles[f].type = type_info[t].type;
   tiles[f].image = type_info[t].image[idx] ;
-  
+
   assert (dep_tree[f].free) ;
   assert (!dep_tree[f].filled) ;
   dep_tree[f].filled = 1 ; /* Lord */
@@ -439,6 +466,106 @@ void place_tile (int f, int t, int idx)
   }
 }
 
+#else
+
+/* This version checks that the tile has a foundation to build on
+ * and that there is a tile to but against either to the left or right.
+ * failing that it checks for a complete free row. */
+static void check_tile_is_free (gint index)
+{
+  int i;
+  gboolean ok;
+  dep_entry * dep;
+
+  dep = dependencies + index;
+
+  if (dep->filled)
+    return;
+  
+  /* First of all check if we're OK with the foundation. */
+  ok = TRUE;
+  for (i=0; i<4; i++) {
+    if (dep->foundation[i] == -1)
+      break;
+    if (!dependencies[dep->foundation[i]].filled) {
+      ok = FALSE;
+      break;
+    }
+  }
+  if (!ok)
+    return;
+
+  /* Now check to the left. */
+  ok = TRUE;
+  for (i=0; i<2; i++) {
+    if (dep->left[i] == -1)
+      break;
+    if (!dependencies[dep->left[i]].filled) {
+      ok = FALSE;
+      break;
+    }
+  }
+  if (ok) {
+    dep->free = TRUE;
+    return;
+  }
+
+  /* And now the right. */
+  ok = TRUE;
+  for (i=0; i<2; i++) {
+    if (dep->right[i] == -1)
+      break;
+    if (!dependencies[dep->right[i]].filled) {
+      ok = FALSE;
+      break;
+    }
+  }
+  if (ok) {
+    dep->free = TRUE;
+  }
+  return;
+}
+
+static void place_tile (int location, int type, int index)
+{
+  gint i;
+  guint target;
+
+  g_print ("(%d) %d, %d, %d\n",location,pos[location].x, pos[location].y, pos[location].layer);
+  
+  /* Set up the tile. */
+  tiles[location].visible = TRUE;
+  tiles[location].selected = FALSE;
+  tiles[location].type = type_info[type].type;
+  tiles[location].image = type_info[type].image[index] ;
+  
+  /* Recalculate dependencies. */
+  dependencies[location].filled = TRUE;
+  dependencies[location].free = FALSE;
+
+  /* See if we've freed anything above us. */
+  for (i=0; i<4; i++) {
+    target = dependencies[location].overhead[i];
+    if (target == -1)
+      break;
+    check_tile_is_free (target);
+  }
+
+  /* See if we've freed anything to the left or right. */
+  for (i=0; i<2; i++) {
+    target = dependencies[location].left[i];
+    if (target != -1)
+      check_tile_is_free (target);
+    target = dependencies[location].right[i];
+    if (target != -1)
+      check_tile_is_free (target);
+  }
+}
+
+#endif
+
+#ifdef USE_CREEPING_HORROR
+
 int tile_free (int tile_num)
 {
   int lp, valid, t, lfree, rfree ;
@@ -481,7 +608,46 @@ int tile_free (int tile_num)
   else return 0 ;
 }
 
-int random_tile_type ()
+#else
+
+int tile_free (int index)
+{
+  dep_entry * dep;
+  int i;
+  gboolean free;
+  
+  dep = dependencies + index;
+
+  /* Check to see we aren't covered. */
+  for (i = 0; i<4; i++) {
+    if ((dep->overhead[i] != -1) && tiles[dep->overhead[i]].visible)
+      return 0;
+  }
+
+  /* Look left. */
+  free = TRUE;
+  for (i = 0; i<2; i++) {
+    if ((dep->left[i] != -1) && tiles[dep->left[i]].visible)
+      free = FALSE;
+  }
+  if (free)
+    return 1;
+
+  /* Look right. */
+  free = TRUE;
+  for (i = 0; i<2; i++) {
+    if ((dep->right[i] != -1) && tiles[dep->right[i]].visible)
+      free = FALSE;
+  }
+
+  return free ? 1 : 0;
+  
+}
+
+#endif
+
+static int
+random_tile_type (void)
 {
   int o ;
   do
@@ -491,7 +657,8 @@ int random_tile_type ()
 }
 
 /* Remove duplicate entries */
-void unique (int *a, int l)
+static void
+unique (int *a, int l)
 {
   int lp, sc ;
   if (l==2)
@@ -515,7 +682,8 @@ void unique (int *a, int l)
 
 /* Generate a single tiles dependancy tree entries */
 /* N.B. Layer increases as you go up tiles BRC is x,y x+right y+down*/
-void tile_deps (int tn)
+static void
+tile_deps (int tn)
 {
   int lp, layer, x, y, lpx, lpy, d ;
 
@@ -560,6 +728,8 @@ void tile_deps (int tn)
   unique(dep_tree[tn].rhs_dep, 2) ;
 }
 
+#ifdef USE_CREEPING_HORROR
+
 /* This calculates the tree of up and down dependancies that is used
    to determine which tiles can be turned / how to place the tiles */
 void generate_dependancies ()
@@ -575,6 +745,79 @@ void generate_dependancies ()
     }
 }
 
+#else
+
+#define generate_dependencies generate_dependancies
+
+void generate_dependencies ()
+{
+  int i,j;
+  int fc, lc, rc, oc;
+  int x,y,l;
+  int tx, ty, tl;
+  dep_entry * dep;
+  
+  dep = dependencies;
+  for (i = 0; i<MAX_TILES; i++) {
+    x = pos[i].x;
+    y = pos[i].y;
+    l = pos[i].layer;
+    dep->free = l == 0;
+    dep->filled = FALSE;
+    
+    dep->foundation[0] = dep->foundation[1] = dep->foundation[2]
+      = dep->foundation[3] = -1;
+    dep->left[0] = dep->left[1] = -1;
+    dep->right[0] = dep->right[1] = -1;
+    dep->overhead[0] = dep->overhead[1] = dep->overhead[2]
+      = dep->overhead[3] = -1;
+    fc = lc = rc = oc = 0;
+    for (j = 0; j<MAX_TILES; j++) {
+      ty = pos[j].y;
+      
+      /* Nothing we are interested in is outside +/- 1 units on the y axis. */
+      if (abs(ty - y) > 1)
+        continue;
+
+      tl = pos[j].layer;
+      tx = pos[j].x;
+        
+      /* First check if it is a foundation tile. */
+      if ((tl == (l - 1)) && (abs(tx - x) < 2)) {
+        dep->foundation[fc++] = j;
+        continue;
+      }
+
+      /* Then do the overhead tiles. */
+      if ((tl == (l + 1)) && (abs(tx - x) < 2)) {
+        dep->overhead[oc++] = j;
+        continue;
+      }
+
+      /* Everything else is on this layer. */
+      if (tl != l)
+        continue;
+      
+      /* Now look left ... */
+      if (tx == (x - 2)) {
+        dep->left[lc++] = j;
+        continue;
+      }
+
+      /* ... and right. */
+      if (tx == (x + 2)) {
+        dep->right[rc++] = j;
+        continue;
+      }      
+    }
+    dep++;
+  }
+}
+
+#endif
+
+#ifdef USE_CREEPING_HORROR
+
 /* Do the tile placement for a soluable game */
 void generate_game (guint seed)
 {
@@ -690,3 +933,82 @@ void generate_game (guint seed)
   global_wait = 1 ;
 #endif
 }
+
+#else
+
+/* If the tile is the first in a row then everything left
+ * and right of it instantly becomes an invalid choice (except
+ * for it's immediate neighbours which are fixed up later). */
+static void clear_row_deps (gint index)
+{
+  int i;
+  dep_entry * d;
+
+  d = dependencies+index;
+
+  if (d->free == FALSE)
+    return;
+  
+  d->free = FALSE;
+
+  for (i = 0; i<2; i++) {
+    if (d->left[i] != -1) {
+      clear_row_deps (d->left[i]);
+    }
+    if (d->right[i] != -1) {
+      clear_row_deps (d->right[i]);
+    }
+  }
+}
+
+static guint get_free_location (GRand * generator)
+{
+  int a,i,j;
+
+  a = g_rand_int_range (generator, 0, MAX_TILES);
+  j = 0;
+
+  for (i=0; i<a; i++) {
+    do {
+      j = (j + 1) % MAX_TILES;
+    } while (!dependencies[j].free);
+  }
+  g_print ("%d, %d\n",a,j);
+  
+  return j;
+}
+
+void generate_game (guint32 seed)
+{
+  GRand * generator;
+  int i, a, b, c, t;
+  
+  generator = g_rand_new_with_seed (seed);
+  
+  for (i=0; i<MAX_TILES/2; i++) {
+    /* Find a random available pair of tiles. */
+    c = g_rand_int_range (generator, 1, MAX_TILES/2 - i + 1);
+    for (t=0; t<MAX_TILES/2; t++)
+      if (type_info[t].placed == 0)
+        if (!(--c)) break;
+
+    type_info[t].placed = 1;
+
+    /* Now find some places to put them. */
+    a = get_free_location (generator);
+    clear_row_deps (a);
+
+    /* There are no collisions because of the preliminary work done
+     * in clear_row_deps. */
+    b = get_free_location (generator);
+    clear_row_deps (b);
+      
+    /* Finally we actually place them. Including the
+     * dependency updates. */
+    place_tile(a,t,0);
+
+    place_tile(b,t,1);
+  }
+}
+
+#endif
