@@ -1,3 +1,5 @@
+/* -*- mode:C; indent-tabs-mode:t; tab-width:8; c-basic-offset:8; -*- */
+
 /*
  * Same-Gnome: the game.
  * (C) 1997 the Free Software Foundation
@@ -12,6 +14,7 @@
 
 #include <stdlib.h>
 #include <time.h>
+#include <string.h>
 
 #include <config.h>
 #include <gnome.h>
@@ -19,7 +22,13 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gconf/gconf-client.h>
 #include <games-gconf.h>
+#include <games-frame.h>
 
+/* Define a sensible alternative to ngettext if we don't have it. Note that
+ * this is only sensible in the context of same-gnome. */
+#ifndef HAVE_NGETTEXT
+#define ngettext(one,lots,n) gettext(lots)
+#endif
 
 #define STONE_SIZE 40
 #define STONE_COLS  15
@@ -30,10 +39,16 @@
 		     GDK_LEAVE_NOTIFY_MASK          |\
 		     GDK_POINTER_MOTION_MASK)
 
+#define KEY_DIR "/apps/same-gnome"
+#define KEY_TILESET "/apps/same-gnome/tileset"
+#define KEY_SCORE "/apps/same-gnome/score"
+#define KEY_NSTONES "/apps/same-gnome/nstones"
+#define KEY_FIELD "/apps/same-gnome/field"
+
 static GtkWidget *pref_dialog, *scorew;
 static GtkWidget *app, *draw_area, *vb, *appbar;
 static GdkPixbuf *image;
-static GdkPixmap *stones, *mask;
+
 static int tagged_count = 0;
 static int ball_timeout_id = -1;
 static int old_x = -1, old_y = -1;
@@ -41,9 +56,8 @@ static int score;
 static gchar *scenario;
 static gint restarted;
 static gboolean game_over = FALSE;
-static gboolean ignore_selection;
 
-void update_score_state ();
+static void update_score_state (void);
 
 /* Prefs */
 GConfClient *conf_client = NULL;
@@ -66,16 +80,24 @@ draw_ball (int x, int y)
 {
 	int bx, by;
 
-	if (field [x][y].color){
+	if (x >= STONE_COLS || x < 0)
+		return;
+	if (y >= STONE_LINES || y < 0)
+		return;
+
+	if (field [x][y].color) {
 		by = STONE_SIZE * (field [x][y].color - 1);
 		bx = STONE_SIZE * (field [x][y].frame);
 
-		gdk_draw_drawable (draw_area->window,
-				 draw_area->style->black_gc, stones,
-				 bx, by, x * STONE_SIZE, y * STONE_SIZE,
-				 STONE_SIZE, STONE_SIZE);
+		gdk_draw_pixbuf (draw_area->window,
+				 draw_area->style->black_gc, image,
+				 bx, by,
+				 x * STONE_SIZE, y * STONE_SIZE,
+				 STONE_SIZE, STONE_SIZE,
+				 GDK_RGB_DITHER_NORMAL, 0, 0);
 	} else {
-		gdk_window_clear_area (draw_area->window, x * STONE_SIZE, y * STONE_SIZE,
+		gdk_window_clear_area (draw_area->window,
+				       x * STONE_SIZE, y * STONE_SIZE,
 				       STONE_SIZE, STONE_SIZE);
 	}
 }
@@ -90,8 +112,8 @@ paint (GdkRectangle *area)
 	x2 = (area->x + area->width) / STONE_SIZE;
 	y2 = (area->y + area->height) / STONE_SIZE;
 
-	for (x = x1; x <= x2; x++){
-		for (y = y1; y <= y2; y++){
+	for (x = x1; x <= x2; x++) {
+		for (y = y1; y <= y2; y++) {
 			draw_ball (x, y);
 		}
 	}
@@ -103,9 +125,9 @@ untag_all ()
 	int x, y;
 
 	for (x = 0; x < STONE_COLS; x++)
-		for (y = 0; y < STONE_LINES; y++){
+		for (y = 0; y < STONE_LINES; y++) {
 			field [x][y].tag   = 0;
-			if (sync_stones){
+			if (sync_stones) {
 				field [x][y].frame = 0;
 				draw_ball (x, y);
 			}
@@ -143,7 +165,7 @@ move_tagged_balls (void *data)
 	int x, y;
 	
 	for (x = 0; x < STONE_COLS; x++)
-		for (y = 0; y < STONE_LINES; y++){
+		for (y = 0; y < STONE_LINES; y++) {
 			if (!field [x][y].tag)
 				continue;
 			field [x][y].frame = (field [x][y].frame + 1) % nstones;
@@ -155,7 +177,7 @@ move_tagged_balls (void *data)
 static void
 disable_timeout ()
 {
-	if (ball_timeout_id != -1){
+	if (ball_timeout_id != -1) {
 		g_source_remove (ball_timeout_id);
 		ball_timeout_id = -1;
 	}
@@ -177,13 +199,14 @@ mark_balls (int x, int y)
 	tagged_count = flood_fill (x, y, field [x][y].color);
 	
 	if (tagged_count > 1) {
-		char *b;
+		gchar *b;
 		ball_timeout_id = g_timeout_add (100, move_tagged_balls, 0);
-		b = g_strdup_printf (_("%d stones selected"), tagged_count);
-                gnome_appbar_set_status (GNOME_APPBAR(appbar), b);
+		b = g_strdup_printf (ngettext ("%d stone selected", "%d stones selected", tagged_count), tagged_count);
+                gnome_appbar_set_status (GNOME_APPBAR (appbar), b);
 		g_free (b);
 	} else
-                gnome_appbar_set_status (GNOME_APPBAR(appbar), _("No stones selected"));
+                gnome_appbar_set_status (GNOME_APPBAR (appbar),
+					 _("No stones selected"));
 }
 
 static void
@@ -191,13 +214,14 @@ compress_column (int x)
 {
 	int y, ym;
 	
-	for (y = STONE_LINES - 1; y >= 0; y--){
-		if (!field [mapx(x)][mapy(y)].tag)
+	for (y = STONE_LINES - 1; y >= 0; y--) {
+		if (!field [mapx (x)][mapy (y)].tag)
 			continue;
 		for (ym = y; ym < STONE_LINES - 1; ym++)
-			field [mapx(x)][mapy(ym)] = field [mapx(x)][mapy(ym+1)];
-		field [mapx(x)][mapy(ym)].color = 0;
-		field [mapx(x)][mapy(ym)].tag   = 0;
+			field [mapx (x)][mapy (ym)] 
+				= field [mapx (x)][mapy (ym+1)];
+		field [mapx (x)][mapy (ym)].color = 0;
+		field [mapx (x)][mapy (ym)].tag   = 0;
 	}
 }
 
@@ -216,7 +240,7 @@ copy_col (int dest, int src)
 	int y;
 	
 	for (y = 0; y < STONE_LINES; y++)
-		field [mapx(dest)][mapy(y)] = field [mapx(src)][mapy(y)];
+		field [mapx (dest)][mapy (y)] = field [mapx (src)][mapy (y)];
 }
 
 static void
@@ -224,9 +248,9 @@ clean_last_col ()
 {
 	int y;
 
-	for (y = 0; y < STONE_LINES; y++){
-		field [mapx(STONE_COLS-1)][mapy(y)].color = 0;
-		field [mapx(STONE_COLS-1)][mapy(y)].tag   = 0;
+	for (y = 0; y < STONE_LINES; y++) {
+		field [mapx (STONE_COLS-1)][mapy (y)].color = 0;
+		field [mapx (STONE_COLS-1)][mapy (y)].tag   = 0;
 	}
 }
 
@@ -235,8 +259,8 @@ compress_x ()
 {
 	int x, xm, l;
 
-	for (x = 0; x < STONE_COLS; x++){
-		for (l = STONE_COLS; field [mapx(x)][mapy(0)].color == 0 && l; l--){
+	for (x = 0; x < STONE_COLS; x++) {
+		for (l = STONE_COLS; field [mapx(x)][mapy(0)].color == 0 && l; l--) {
 			for (xm = x; xm < STONE_COLS-1; xm++)
 				copy_col (xm, xm+1);
 			clean_last_col ();
@@ -251,7 +275,7 @@ set_score (int new_score)
 	
 	score = new_score;
 	b = g_strdup_printf ("%.5d", score);
-	gtk_label_set_text (GTK_LABEL(scorew), b);
+	gtk_label_set_text (GTK_LABEL (scorew), b);
 	g_free (b);
 }
 
@@ -260,17 +284,19 @@ show_scores (guint pos)
 {
 	GtkWidget *dialog;
 
-	dialog = gnome_scores_display (_("The Same GNOME"), "same-gnome", NULL, pos);
+	dialog = gnome_scores_display (_("The Same GNOME"),
+				       "same-gnome", NULL, pos);
 	if (dialog != NULL) {
-		gtk_window_set_transient_for (GTK_WINDOW(dialog), GTK_WINDOW(app));
-		gtk_window_set_modal (GTK_WINDOW(dialog), TRUE);
+		gtk_window_set_transient_for (GTK_WINDOW (dialog),
+					      GTK_WINDOW (app));
+		gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
 	}
 }
 
 static void
-game_top_ten_callback(GtkWidget *widget, gpointer data)
+game_top_ten_callback (GtkWidget *widget, gpointer data)
 {
-	show_scores(0);
+	show_scores (0);
 }
 
 static void
@@ -280,32 +306,32 @@ check_game_over (void)
 	int x,y;
 	int pos;
 	
-	for(x = 0; x < STONE_COLS; x++)
-		for(y = 0 ; y < STONE_LINES; y++) {
+	for (x = 0; x < STONE_COLS; x++)
+		for (y = 0 ; y < STONE_LINES; y++) {
 			if (!field [x][y].color)
 				continue;
 			cleared = 0;
-			if(x+1 < STONE_COLS) 
+			if (x+1 < STONE_COLS) 
 				if(field[x][y].color == field[x+1][y].color)
 					return;
-			if(y+1 < STONE_LINES) 
-				if(field[x][y].color == field[x][y+1].color)
+			if (y+1 < STONE_LINES) 
+				if (field[x][y].color == field[x][y+1].color)
 					return;
 		}
 
 	if (cleared)
 		set_score (score+1000);
 
-	pos = gnome_score_log(score, NULL, TRUE);
+	pos = gnome_score_log (score, NULL, TRUE);
 	update_score_state ();
-	show_scores(pos);
+	show_scores (pos);
 	game_over = TRUE;
 }
 
 static void
 kill_balls (int x, int y)
 {
-	if (!field [x][y].color)
+	if (! field [x][y].color)
 		return;
 	
 	if (tagged_count < 2)
@@ -314,14 +340,14 @@ kill_balls (int x, int y)
 	set_score (score + (tagged_count - 2) * (tagged_count - 2));
 	compress_y ();
 	compress_x ();
-	gtk_widget_draw (draw_area, NULL);
+	gtk_widget_queue_draw (GTK_WIDGET (draw_area));
 	check_game_over ();
 }
 
 static gint
 area_event (GtkWidget *widget, GdkEvent *event, void *d)
 {
-	switch (event->type){
+	switch (event->type) {
 	case GDK_EXPOSE: {
 		GdkEventExpose *e = (GdkEventExpose *) event;
 		paint (&e->area);
@@ -364,7 +390,7 @@ fill_board (void)
 	int x, y;
 
 	for (x = 0; x < STONE_COLS; x++)
-		for (y = 0; y < STONE_LINES; y++){
+		for (y = 0; y < STONE_LINES; y++) {
 			field [x][y].color = 1 + (rand () % ncolors);
 			field [x][y].tag   = 0;
 			field [x][y].frame = sync_stones ? 0 : (rand () % nstones);
@@ -377,13 +403,13 @@ new_game (void)
 	game_over = FALSE;
 	fill_board ();
 	set_score (0);
-	gtk_widget_draw (draw_area, NULL);
+	gtk_widget_queue_draw (GTK_WIDGET (draw_area));
 }
 
 static void
 configure_sync (char *fname)
 {
-	if (strstr (fname, "-sync.png"))
+	if (g_strrstr (fname, "-sync.png"))
 		sync_stones = 1;
 	else
 		sync_stones = 0;
@@ -392,24 +418,30 @@ configure_sync (char *fname)
 static void
 load_scenario (char *fname)
 {
-	char *tmp, *fn;
-        GdkColor bgcolor;
-        GdkImage *tmpimage;
-	int i, j;
+	gchar *fn;
+        GdkColor background_color;
+	gint i, j;
+	GdkColormap *colormap;
+	gchar *color_name;
     
 	g_return_if_fail (fname != NULL);
 
-	fn = g_strconcat ( PIXMAPDIR, "/", fname, NULL);
+	if (g_path_is_absolute (fname))
+		fn = g_strdup (fname);
+	else
+		fn = g_build_filename (PIXMAPDIR, fname, NULL);
 
-	if (!g_file_test (fn, G_FILE_TEST_EXISTS)) {
-		GtkWidget *box = gtk_message_dialog_new (GTK_WINDOW (app),
-			GTK_DIALOG_DESTROY_WITH_PARENT,
-			GTK_MESSAGE_ERROR,
-			GTK_BUTTONS_OK,
-			_("Could not find the theme:\n%s\n\n"
-			"Please check your Same GNOME installation."), fn);
+	if (! g_file_test (fn, G_FILE_TEST_EXISTS)) {
+		GtkWidget *box = gtk_message_dialog_new 
+			(GTK_WINDOW (app),
+			 GTK_DIALOG_DESTROY_WITH_PARENT,
+			 GTK_MESSAGE_ERROR,
+			 GTK_BUTTONS_OK,
+			 _("Could not find the theme:\n%s\n\n"
+			   "Please check your Same GNOME installation."), fn);
 
 		gtk_dialog_set_default_response (GTK_DIALOG (box), GTK_RESPONSE_OK);
+                gtk_dialog_set_has_separator (GTK_DIALOG (box), FALSE);
 		gtk_dialog_run (GTK_DIALOG (box));
 		gtk_widget_destroy (box);
                 /* We may not have called gtk_main yet, but if we do we want
@@ -421,24 +453,27 @@ load_scenario (char *fname)
 	}
 
 	g_free (scenario);
-	scenario = g_strdup(fname);
+	scenario = g_strdup (fname);
 
 	configure_sync (fname);
 
 	if (image)
-		gdk_pixbuf_unref (image);
+		g_object_unref (image);
 
 	image = gdk_pixbuf_new_from_file (fn, NULL);
 
 	if (image == NULL) {
-		GtkWidget *box = gtk_message_dialog_new (GTK_WINDOW (app),
-			GTK_DIALOG_DESTROY_WITH_PARENT,
-			GTK_MESSAGE_ERROR,
-			GTK_BUTTONS_OK,
-			_("Same GNOME can't load the image file:\n%s\n\n"
-			 "Please check your Same GNOME installation"), fn);
-
-		gtk_dialog_set_default_response (GTK_DIALOG (box), GTK_RESPONSE_OK);
+		GtkWidget *box = gtk_message_dialog_new 
+			(GTK_WINDOW (app),
+			 GTK_DIALOG_DESTROY_WITH_PARENT,
+			 GTK_MESSAGE_ERROR,
+			 GTK_BUTTONS_OK,
+			 _("Same GNOME can't load the image file:\n%s\n\n"
+			   "Please check your Same GNOME installation"), fn);
+		
+		gtk_dialog_set_default_response (GTK_DIALOG (box),
+						 GTK_RESPONSE_OK);
+                gtk_dialog_set_has_separator (GTK_DIALOG (box), FALSE);
 		gtk_dialog_run (GTK_DIALOG (box));
 		gtk_widget_destroy (box);
                 /* We may not have called gtk_main yet, but if we do we want
@@ -449,12 +484,15 @@ load_scenario (char *fname)
                   exit(1);
 	}
 
-	gdk_pixbuf_render_pixmap_and_mask (image, &stones, &mask, 127);
+	/* FIXME: this should be a preference */
+	color_name = g_strdup ("#000000");
+	gdk_color_parse (color_name, &background_color);
+	g_free (color_name);
 
-        tmpimage = gdk_drawable_get_image (stones, 0, 0, 1, 1);
-        bgcolor.pixel = gdk_image_get_pixel(tmpimage, 0, 0);
-        gdk_window_set_background (draw_area->window, &bgcolor);
-	gdk_image_unref (tmpimage);
+	colormap = gtk_widget_get_colormap (draw_area);
+	gdk_colormap_alloc_color (colormap, &background_color, FALSE, TRUE);
+
+        gdk_window_set_background (draw_area->window, &background_color);
   
 	g_free( fn );
 
@@ -474,7 +512,7 @@ load_scenario (char *fname)
 	ncolors = 3;
 
 
-	gtk_widget_draw (draw_area, NULL);
+	gtk_widget_queue_draw (GTK_WIDGET (draw_area));
 }
 
 static void
@@ -484,17 +522,17 @@ create_same_board (char *fname)
 
 	gtk_widget_set_events (draw_area, gtk_widget_get_events (draw_area) | GAME_EVENTS);
 
-	gtk_box_pack_start_defaults (GTK_BOX(vb), draw_area);
+	gtk_box_pack_start_defaults (GTK_BOX (vb), draw_area);
 	gtk_widget_realize (draw_area);
   
 	gtk_widget_show (draw_area);
 
 	load_scenario (fname);
-	gtk_drawing_area_size (GTK_DRAWING_AREA (draw_area),
-			       STONE_COLS  * STONE_SIZE,
-			       STONE_LINES * STONE_SIZE);
+	gtk_widget_set_size_request (GTK_WIDGET (draw_area),
+				     STONE_COLS  * STONE_SIZE,
+				     STONE_LINES * STONE_SIZE);
 	g_signal_connect (G_OBJECT (draw_area), "event",
-			  G_CALLBACK(area_event), 0);
+			  G_CALLBACK (area_event), 0);
 }
 
 static void
@@ -510,18 +548,11 @@ set_selection (GtkTreeSelection *selection, gpointer data)
         GtkTreeModel * model;
         GtkTreeIter iter;
 
-        /* This is here because I can't figure out how to set the list
-         * so nothing is selected when it is created. */
-        if (ignore_selection) {
-                ignore_selection = FALSE;
-                return;
-        }
-
         gtk_tree_selection_get_selected (selection, &model, &iter);
         gtk_tree_model_get (model, &iter, 1, &filename, -1);
 
 	load_scenario (filename);
-	gconf_client_set_string (conf_client, "/apps/same-gnome/tileset", filename, NULL);
+	gconf_client_set_string (conf_client, KEY_TILESET, filename, NULL);
 }
 
 static void
@@ -536,21 +567,23 @@ fill_list (GtkListStore *list)
 	if (!dir)
 		return;
 	
-	while ((filename = g_strdup (g_dir_read_name (dir))) != NULL){
+	while ((filename = g_strdup (g_dir_read_name (dir))) != NULL) {
 		gchar *name;
                 gchar *suffix;
-                
-                if (!g_file_test (filename, G_FILE_TEST_IS_REGULAR)) {
-                        g_free(filename);
+                gchar *path = NULL;
+                path = g_build_filename (PIXMAPDIR, filename, NULL);
+                if (!g_file_test (path, G_FILE_TEST_IS_REGULAR)) {
+                        g_free (filename);
+			g_free (path);
                         continue;
                 }
 
                 /* We strip any trailing suffix, any -sync suffix
                  * and convert '_' to ' '. This is brutal code. */
-                name = g_strdup(filename);
-                suffix = g_strrstr(name,".");
+                name = g_strdup (filename);
+                suffix = g_strrstr (name,".");
                 if (suffix) *suffix = '\0';
-                suffix = g_strrstr(name,"-sync");
+                suffix = g_strrstr (name,"-sync");
                 if (suffix) *suffix = '\0';
                 suffix = name;
                 while (*suffix) {
@@ -575,7 +608,7 @@ static void
 game_preferences_callback (GtkWidget *widget, void *data)
 {
 	GtkWidget *listview, *frame;
-	GtkWidget *button, *scroll;
+	GtkWidget *scroll;
         GtkTreeViewColumn *column;
         GtkTreeSelection * select;
         GtkListStore *list;
@@ -585,8 +618,6 @@ game_preferences_callback (GtkWidget *widget, void *data)
 		return;
 	}
 
-        ignore_selection = TRUE;
-        
 	pref_dialog = gtk_dialog_new_with_buttons (_("Same GNOME Preferences"),
 			GTK_WINDOW (app),
 			GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
@@ -595,6 +626,7 @@ game_preferences_callback (GtkWidget *widget, void *data)
 
 	gtk_dialog_set_default_response (GTK_DIALOG (pref_dialog),
 					 GTK_RESPONSE_OK);
+        gtk_dialog_set_has_separator (GTK_DIALOG (pref_dialog), FALSE);
 	g_signal_connect (G_OBJECT (pref_dialog), "response",
 			  G_CALLBACK(pref_dialog_response), NULL);
 
@@ -613,6 +645,36 @@ game_preferences_callback (GtkWidget *widget, void *data)
         select = gtk_tree_view_get_selection (GTK_TREE_VIEW (listview));
         gtk_tree_selection_set_mode (select, GTK_SELECTION_BROWSE);
         gtk_tree_selection_unselect_all (select);
+
+	{
+		gboolean valid;
+		GtkTreeIter iter;
+		GtkTreeModel *model;
+		gint index;
+
+		/* Get an iter associated with the view */
+		
+		model = gtk_tree_view_get_model (GTK_TREE_VIEW (listview));
+		valid = gtk_tree_model_get_iter_first (model, &iter);
+		index = 0;
+		while (valid) {
+			/* Walk through list, testing each row */
+			gchar *filename;
+			gtk_tree_model_get (model, &iter,
+					    1, &filename, -1);
+
+			if (strcmp (filename, scenario) == 0) {
+				gtk_tree_view_set_cursor (GTK_TREE_VIEW (listview),
+							  gtk_tree_path_new_from_indices (index, -1),
+							  NULL, FALSE);
+				/*gtk_tree_selection_select_iter (select,
+				  &iter);*/
+			}
+			valid = gtk_tree_model_iter_next (model, &iter);
+			index++;
+		}
+	}
+
         g_signal_connect (G_OBJECT (select), "changed",
                           G_CALLBACK (set_selection), NULL);
         
@@ -623,28 +685,26 @@ game_preferences_callback (GtkWidget *widget, void *data)
         gtk_widget_set_size_request (scroll, 250, 200);
         gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scroll),
                                              GTK_SHADOW_ETCHED_IN);
-        gtk_container_add (GTK_CONTAINER(scroll), listview);
-        gtk_widget_set (scroll, "border-width", GNOME_PAD, NULL);
+        gtk_container_add (GTK_CONTAINER (scroll), listview);
         
-        frame = gtk_frame_new (_("Theme"));
+        frame = games_frame_new (_("Theme"));
         gtk_container_add (GTK_CONTAINER (frame), scroll);
-        gtk_widget_set (frame, "border-width", GNOME_PAD_SMALL, NULL);
 
-        gtk_box_pack_start (GTK_BOX (GTK_DIALOG(pref_dialog)->vbox),
+        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (pref_dialog)->vbox),
                             frame, TRUE, TRUE, GNOME_PAD_SMALL);
 
         gtk_widget_show_all (pref_dialog);
 }
 
-static int
+static void
 game_about_callback (GtkWidget *widget, void *data)
 {
 	GdkPixbuf *pixbuf = NULL;
 	static GtkWidget *about = NULL;
 	const gchar *authors[] = {
-		"Miguel de Icaza.",
-		"Federico Mena.",
-		"Horacio J. Peña.",
+		"Miguel de Icaza",
+		"Federico Mena",
+		"Horacio J. Pe\xc3\xb1""a",		
 		NULL
 	};
 	gchar *documenters[] = {
@@ -671,23 +731,22 @@ game_about_callback (GtkWidget *widget, void *data)
 		}
 	}
 	
-	about = gnome_about_new (_("The Same GNOME"), VERSION,
-				 "(C) 1997-1998 the Free Software Foundation",
-				 _("Original idea from KDE's same game program."),
-				 (const char **)authors,
+	about = gnome_about_new (_("Same GNOME"), VERSION,
+				 "Copyright \xc2\xa9 1997-2003 Free Software "
+				 "Foundation, Inc.",
+				 _("Original idea from KDE's same game "
+				   "program."),				 (const char **)authors,
 				 (const char **)documenters,
 				 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
                                  pixbuf);
 	if (pixbuf != NULL)
-		gdk_pixbuf_unref (pixbuf);
+		g_object_unref (pixbuf);
 	
 	gtk_window_set_transient_for (GTK_WINDOW (about), GTK_WINDOW (app));
 	gtk_widget_show (about);
 
 	g_signal_connect (G_OBJECT (about), "destroy",
 			  G_CALLBACK (gtk_widget_destroyed), &about);
-
-	return TRUE;
 }
 
 static int
@@ -733,7 +792,8 @@ GnomeUIInfo mainmenu[] = {
 	GNOMEUIINFO_END
 };
 
-void update_score_state ()
+static void 
+update_score_state (void)
 {
         gchar **names = NULL;
         gfloat *scores = NULL;
@@ -760,29 +820,22 @@ save_state (GnomeClient *client,
 	    gint fast,
 	    gpointer client_data)
 {
-	gchar *argv []= { "rm", "-r", NULL };
 	gchar *buf;
 	struct ball *f = (struct ball*) field;
 	int i;  
 	
-	gconf_client_set_int (conf_client, "/apps/same-gnome/score", score, NULL);
-	gconf_client_set_int (conf_client, "/apps/same-gnome/nstones", sync_stones ? 1 : nstones, NULL);
+	gconf_client_set_int (conf_client, KEY_SCORE, score, NULL);
+	gconf_client_set_int (conf_client, KEY_NSTONES, sync_stones ? 1 : nstones, NULL);
 	
 	buf= g_malloc (STONE_COLS*STONE_LINES+1);
 
-	for (i = 0 ; i < (STONE_COLS*STONE_LINES); i++){
+	for (i = 0 ; i < (STONE_COLS*STONE_LINES); i++) {
 		buf [i]= f [i].color + 'a';
 	}
 	buf [STONE_COLS*STONE_LINES]= '\0';
-	gconf_client_set_string (conf_client, "/apps/same-gnome/field", buf, NULL);
-	g_free(buf);
+	gconf_client_set_string (conf_client, KEY_FIELD, buf, NULL);
+	g_free (buf);
 
-	/* FIXME if anybody knows what this does...
-	argv[2]= gnome_config_get_real_path (prefix);
-	gnome_client_set_discard_command (client, 3, argv);
-	gnome_client_set_discard_command (client, 2, argv);
-	*/
-	
 	return TRUE;
 }
 
@@ -794,10 +847,10 @@ restart (void)
 	struct ball *f = (struct ball*) field;
 	int i;
 	
-	score = gconf_client_get_int (conf_client, "/apps/same-gnome/score", NULL);
-	nstones = gconf_client_get_int (conf_client, "/apps/same-gnome/nstones", NULL);
+	score = gconf_client_get_int (conf_client, KEY_SCORE, NULL);
+	nstones = gconf_client_get_int (conf_client, KEY_NSTONES, NULL);
 	
-	buf = gconf_client_get_string (conf_client, "/apps/same-gnome/field", NULL);
+	buf = gconf_client_get_string (conf_client, KEY_FIELD, NULL);
 
 	if (buf) {
 		for (i= 0; i < (STONE_COLS*STONE_LINES); i++) 
@@ -857,48 +910,48 @@ main (int argc, char *argv [])
 	g_signal_connect (G_OBJECT (client), "die",
 			  G_CALLBACK (client_die), NULL);
 
-	if (GNOME_CLIENT_RESTARTED (client)){
+	if (GNOME_CLIENT_RESTARTED (client)) {
 		restart ();
 		restarted = 1;
 	}
 
 	/* Get the default GConfClient */
 	conf_client = gconf_client_get_default ();
-	if (!games_gconf_sanity_check_string (conf_client, "/apps/same-gnome/tileset")) {
+	if (!games_gconf_sanity_check_string (conf_client, KEY_TILESET)) {
 		return 1;
 	}
 
 	srand (time (NULL));
 
-	app = gnome_app_new("same-gnome", _("Same GNOME"));
+	app = gnome_app_new ("same-gnome", _("Same GNOME"));
 
-        gtk_window_set_policy(GTK_WINDOW(app), FALSE, FALSE, TRUE);
+        gtk_window_set_resizable (GTK_WINDOW (app), FALSE);
 	g_signal_connect (G_OBJECT (app), "delete_event",
-			  G_CALLBACK(game_quit_callback), NULL);
+			  G_CALLBACK (game_quit_callback), NULL);
 
-	appbar = gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_USER);
-	gnome_app_set_statusbar(GNOME_APP (app), GTK_WIDGET(appbar));
+	appbar = gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_USER);
+	gnome_app_set_statusbar (GNOME_APP (app), GTK_WIDGET (appbar));
 
-	gnome_appbar_set_status(GNOME_APPBAR (appbar),
+	gnome_appbar_set_status (GNOME_APPBAR (appbar),
 				_("Welcome to Same GNOME!"));
 
-	gnome_app_create_menus(GNOME_APP(app), mainmenu);
+	gnome_app_create_menus (GNOME_APP (app), mainmenu);
 
-	gnome_app_install_menu_hints(GNOME_APP (app), mainmenu);
+	gnome_app_install_menu_hints (GNOME_APP (app), mainmenu);
   
         vb = gtk_vbox_new (FALSE, 0);
 	gnome_app_set_contents (GNOME_APP (app), vb);
 
-	if (!fname) {
+	if (! fname) {
 		fname = gconf_client_get_string
-			(conf_client, "/apps/same-gnome/tileset", NULL);
+			(conf_client, KEY_TILESET, NULL);
 	}
 
 	create_same_board (fname);
 
 	update_score_state ();
 	
-	label = gtk_label_new (_("Score: "));
+	label = gtk_label_new (_("Score:"));
 	scorew = gtk_label_new ("");
 	set_score (score);
 
